@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2019 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -52,6 +52,7 @@
 #include "magick/monitor.h"
 #include "magick/profile.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declaractions.
@@ -115,7 +116,7 @@ typedef struct _CineonImageInfo
 {
   U8    orientation;               /* Image orientation */
   U8    channels;                  /* Number of image channels (1-8) */
-  U8    pad[2];                    /* Unused (2 byte space for word allignment */
+  U8    pad[2];                    /* Unused (2 byte space for word alignment */
   DPXImageChannel channel_info[8]; /* Description of channels */
   R32   white_point[2];            /* White point (color temperature) - x,y pair */
   R32   red_primary_chromaticity[2]; /* Red primary chromaticity - x,y pair */
@@ -154,7 +155,7 @@ typedef struct _CineonFilmInfo
   U8    film_mfg_id_code;          /* Film mfg. ID code (2 digits from film edge code) */
   U8    film_type;                 /* Film type (2 digits from film edge code) */
   U8    perfs_offset;              /* Offset in perfs (2 digits from film edge code) */
-  U8    unused;                    /* Unused, for word allignment */
+  U8    unused;                    /* Unused, for word alignment */
   U32   prefix;                    /* Prefix (6 digits from film edge code) */
   U32   count;                     /* Count (4 digits from film edge code) */
   ASCII format[32];                /* Format -- e.g. Academy */
@@ -555,7 +556,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                         "Creation time \"%s\"", cin_file_info.creation_time);
   /*
-    Place arbitary limit on user defined length.
+    Place arbitrary limit on user defined length.
   */
   if (cin_file_info.user_defined_length > 16777216U)
     ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
@@ -717,13 +718,18 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
       user_data=(unsigned char *) NULL;
       while (user_data_length < cin_file_info.user_defined_length)
         {
+          unsigned char *new_user_data;
           read_size=Min(block_size,cin_file_info.user_defined_length-user_data_length);
-          MagickReallocMemory(unsigned char *,user_data,user_data_length+read_size);
-          if (user_data == (unsigned char *) NULL)
-            ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+          new_user_data=MagickReallocateResourceLimitedMemory(unsigned char *,user_data,user_data_length+read_size);
+          if (new_user_data == (unsigned char *) NULL)
+            {
+              MagickFreeResourceLimitedMemory(unsigned char *,user_data);
+              ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+            }
+          user_data=new_user_data;
           if (ReadBlob(image,read_size,user_data+user_data_length) != read_size)
             {
-              MagickFreeMemory(user_data);
+              MagickFreeResourceLimitedMemory(unsigned char *,user_data);
               ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
             }
           user_data_length += read_size;
@@ -731,10 +737,10 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
         }
       if (!SetImageProfile(image,"CINEONUSERDATA",user_data,user_data_length))
         {
-          MagickFreeMemory(user_data);
+          MagickFreeResourceLimitedMemory(unsigned char *,user_data);
           ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
         }
-      MagickFreeMemory(user_data);
+      MagickFreeResourceLimitedMemory(unsigned char *,user_data);
     }
 
   if (image_info->ping)
@@ -759,16 +765,16 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
   if (BlobIsSeekable(image))
     {
       magick_off_t file_size;
-      magick_int64_t expected_file_size;
+      magick_intmax_t expected_file_size;
 
       file_size = GetBlobSize(image);
 
       expected_file_size=pixels_offset+
-        ((((((magick_int64_t) image->rows*image->columns*number_of_channels+2UL)/3UL)
+        ((((((magick_intmax_t) image->rows*image->columns*number_of_channels+2UL)/3UL)
            *sizeof(U32)*8UL)+31UL)/32UL)*sizeof(U32);
       if (image->logging)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                              "Expected file size %" MAGICK_UINT64_F "u bytes"
+                              "Expected file size %" MAGICK_UINTMAX_F "u bytes"
                               " (have %" MAGICK_OFF_F "d bytes)",
                               expected_file_size, file_size);
 
@@ -796,11 +802,11 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
         {
           scandata_bytes=4;
           scale_to_short=64;
-          scandata=MagickAllocateMemory(unsigned char *,scandata_bytes);
+          scandata=MagickAllocateResourceLimitedMemory(unsigned char *,scandata_bytes);
           if (scandata == (unsigned char *) NULL)
             ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
           scanline=scandata;
-          MagickBitStreamInitializeRead(&bit_stream,scanline);
+          MagickBitStreamInitializeRead(&bit_stream,(const unsigned char*) scanline);
           for (y=0; y < image->rows; y++)
             {
               q=SetImagePixels(image,0,y,image->columns,1);
@@ -819,7 +825,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
                       if (ReadBlobZC(image,scandata_bytes,&scanline) !=
                           scandata_bytes)
                         break;
-                      MagickBitStreamInitializeRead(&bit_stream,scanline);
+                      MagickBitStreamInitializeRead(&bit_stream,(const unsigned char *) scanline);
                       i=0;
                     }
                   q->red=q->green=q->blue=
@@ -838,14 +844,14 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
                                               image->columns,image->rows))
                     break;
             }
-          MagickFreeMemory(scandata);
+          MagickFreeResourceLimitedMemory(unsigned char *,scandata);
           break;
         }
       case 3:
         {
           scandata_bytes=MagickArraySize(image->columns,4);
           scale_to_short=64;
-          scandata=MagickAllocateMemory(unsigned char *,scandata_bytes);
+          scandata=MagickAllocateResourceLimitedMemory(unsigned char *,scandata_bytes);
           if (scandata == (unsigned char *) NULL)
             ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
           for (y=0; y < image->rows; y++)
@@ -860,7 +866,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
               scanline=scandata;
               if (ReadBlobZC(image,scandata_bytes,&scanline) != scandata_bytes)
                 break;
-              MagickBitStreamInitializeRead(&bit_stream,scanline);
+              MagickBitStreamInitializeRead(&bit_stream,(const unsigned char*) scanline);
               for (x=0 ; x < image->columns; x++)
                 {
                   /*
@@ -891,7 +897,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
                                               image->columns,image->rows))
                     break;
             }
-          MagickFreeMemory(scandata);
+          MagickFreeResourceLimitedMemory(unsigned char *,scandata);
           break;
         }
       default:
@@ -930,6 +936,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
   if (EOFBlob(image))
     ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,image->filename);
   CloseBlob(image);
+  StopTimer(&image->timer);
   return(GetFirstImageInList(image));
 }
 
@@ -1078,9 +1085,9 @@ ModuleExport void UnregisterCINEONImage(void)
     *definition_value; \
 \
   if ((definition_value=AccessDefinition(image_info,"dpx",&key[4]))) \
-    member.f=strtod(definition_value, (char **) NULL); \
+    member.f=(float) strtod(definition_value, (char **) NULL); \
   else if ((attribute=GetImageAttribute(image,key))) \
-    member.f=strtod(attribute->value, (char **) NULL); \
+    member.f=(float) strtod(attribute->value, (char **) NULL); \
   else \
     SET_UNDEFINED_R32(member); \
 }
@@ -1111,19 +1118,28 @@ static void GenerateCineonTimeStamp(char *date_str, size_t date_str_length, char
   time_t
     current_time;
 
+#if defined(HAVE_LOCALTIME_R)
+  struct tm
+    tm_buf;
+#endif /* if defined(HAVE_LOCALTIME_R) */
+
   const struct tm
     *t;
 
   current_time=time((time_t *) NULL);
-  t=localtime(&current_time);
+#if defined(HAVE_LOCALTIME_R)
+  t=localtime_r(&current_time, &tm_buf);
+#else
+  t=localtime(&current_time); /* Thread-unsafe version */
+#endif  /* if defined(HAVE_LOCALTIME_R) */
 
   (void) memset(timestamp,0,sizeof(timestamp));
   (void) strftime(timestamp,MaxTextExtent,"%Y:%m:%d:%H:%M:%S%Z",t);
   timestamp[MaxTextExtent-1]='\0';
   (void) memset(date_str,0,date_str_length);
-  (void) strlcpy(date_str,timestamp,11);
+  (void) strlcpy(date_str,timestamp,date_str_length);
   (void) memset(time_str,0,time_str_length);
-  (void) strlcpy(time_str,timestamp+11,15);
+  (void) strlcpy(time_str,timestamp+11,time_str_length);
 }
 
 
@@ -1256,32 +1272,32 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
   SET_UNDEFINED_R32(cin_image_info.white_point[1]);
   if ( image->chromaticity.white_point.x != 0.0 && image->chromaticity.white_point.y != 0.0 )
     {
-      cin_image_info.white_point[0].f = image->chromaticity.white_point.x;
-      cin_image_info.white_point[1].f = image->chromaticity.white_point.y;
+      cin_image_info.white_point[0].f = (float) image->chromaticity.white_point.x;
+      cin_image_info.white_point[1].f = (float) image->chromaticity.white_point.y;
     }
   /* Red primary chromaticity - x,y pair */
   SET_UNDEFINED_R32(cin_image_info.red_primary_chromaticity[0]);
   SET_UNDEFINED_R32(cin_image_info.red_primary_chromaticity[1]);
   if ( image->chromaticity.red_primary.x != 0.0 &&  image->chromaticity.red_primary.y != 0.0)
     {
-      cin_image_info.red_primary_chromaticity[0].f = image->chromaticity.red_primary.x;
-      cin_image_info.red_primary_chromaticity[1].f = image->chromaticity.red_primary.y;
+      cin_image_info.red_primary_chromaticity[0].f = (float) image->chromaticity.red_primary.x;
+      cin_image_info.red_primary_chromaticity[1].f = (float) image->chromaticity.red_primary.y;
     }
   /* Green primary chromaticity - x,y pair */
   SET_UNDEFINED_R32(cin_image_info.green_primary_chromaticity[0]);
   SET_UNDEFINED_R32(cin_image_info.green_primary_chromaticity[1]);
   if ( image->chromaticity.green_primary.x != 0.0 && image->chromaticity.green_primary.y != 0.0 )
     {
-      cin_image_info.green_primary_chromaticity[0].f = image->chromaticity.green_primary.x;
-      cin_image_info.green_primary_chromaticity[1].f = image->chromaticity.green_primary.y;
+      cin_image_info.green_primary_chromaticity[0].f = (float) image->chromaticity.green_primary.x;
+      cin_image_info.green_primary_chromaticity[1].f = (float) image->chromaticity.green_primary.y;
     }
   /* Blue primary chromaticity - x,y pair */
   SET_UNDEFINED_R32(cin_image_info.blue_primary_chromaticity[0]);
   SET_UNDEFINED_R32(cin_image_info.blue_primary_chromaticity[1]);
   if ( image->chromaticity.blue_primary.x != 0.0 && image->chromaticity.blue_primary.y != 0.0 )
     {
-      cin_image_info.blue_primary_chromaticity[0].f = image->chromaticity.blue_primary.x;
-      cin_image_info.blue_primary_chromaticity[1].f = image->chromaticity.blue_primary.y;
+      cin_image_info.blue_primary_chromaticity[0].f = (float) image->chromaticity.blue_primary.x;
+      cin_image_info.blue_primary_chromaticity[1].f = (float) image->chromaticity.blue_primary.y;
     }
   /* Label text */
   AttributeToString(image_info,image,"DPX:file.project.name",cin_image_info.label_text);
@@ -1466,7 +1482,7 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
     scale_from_short=(65535U / (65535U >> (16-bits_per_sample)));
 
     scanline_bytes=MagickArraySize(image->columns,4);
-    scanline=MagickAllocateMemory(unsigned char *,scanline_bytes);
+    scanline=MagickAllocateResourceLimitedMemory(unsigned char *,scanline_bytes);
     if (scanline == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     (void) memset(scanline,0,scanline_bytes);
@@ -1509,7 +1525,7 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
                                         image->columns,image->rows))
               break;
       }
-    MagickFreeMemory(scanline);
+    MagickFreeResourceLimitedMemory(unsigned char *,scanline);
   }
 
   if ((magick_off_t) cin_file_info.file_size != TellBlob(image))
@@ -1518,7 +1534,13 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
                     cin_file_info.file_size,
                     (unsigned int) TellBlob(image));
     }
+  if ((magick_off_t) offset != TellBlob(image))
+    {
+      (void) printf("### Offset %lu, TellBlob says %u\n",
+                    (unsigned long) offset,
+                    (unsigned int) TellBlob(image));
+    }
 
-  CloseBlob(image);
+  status &= CloseBlob(image);
   return(status);
 }
