@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2022 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -45,6 +45,7 @@
 #include "magick/render.h"
 #include "magick/tempfile.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -122,6 +123,7 @@ static unsigned int IsEPT(const unsigned char *magick,const size_t length)
 %
 %
 */
+#if defined(HasGS)
 static Image *ReadEPTImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -160,7 +162,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   unsigned int
     antialias=4;
 
-  ExtendedSignedIntegralType
+  magick_uint32_t
     filesize;
 
   RectangleInfo
@@ -170,13 +172,13 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   register char
     *p;
 
-  register long
+  register size_t
     i;
 
   SegmentInfo
     bounds;
 
-  size_t
+  magick_uint32_t
     count;
 
   unsigned long
@@ -228,13 +230,15 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   (void) ReadBlobLSBLong(image);
   count=ReadBlobLSBLong(image);
   filesize=ReadBlobLSBLong(image);
-  if (image->logging)
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "File Size: %lu,  Offset: %lu",
-                          (unsigned long) filesize, (unsigned long) count);
   if (EOFBlob(image))
     ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
-  for (i=0; i < (long) (count-12); i++)
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "File Size: %u,  Offset: %u",
+                          (unsigned int) filesize, (unsigned int) count);
+  if ((count <= 12) || (filesize == 0))
+    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+  for (i=0; i < (count-12); i++)
     if (ReadBlobByte(image) == EOF)
       ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
   /*
@@ -255,7 +259,11 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                           "Copying Postscript to temporary file \"%s\" ...",
                           postscript_filename);
-  for (i=0; i < (long) filesize; i++)
+  bounds.x1=0.0;
+  bounds.y1=0.0;
+  bounds.x2=0.0;
+  bounds.y2=0.0;
+  for (i=0; i < filesize; i++)
   {
     if ((c=ReadBlobByte(image)) == EOF)
       break;
@@ -381,23 +389,13 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
         *clone_info;
 
       clone_info=CloneImageInfo(image_info);
-      clone_info->blob=(void *) NULL;
+      clone_info->blob=(_BlobInfoPtr_) NULL;
       clone_info->length=0;
       clone_info->magick[0]='\0';
       image=ReadImage(clone_info,exception);
       DestroyImageInfo(clone_info);
     }
   (void) LiberateTemporaryFile((char *) image_info->filename);
-#if defined(HasDPS)
-  if (image == (Image *) NULL)
-    {
-      /*
-        Ghostscript has failed-- try the Display Postscript Extension.
-      */
-      (void) FormatString((char *) image_info->filename,"dps:%.1024s",filename);
-      image=ReadImage(image_info,exception);
-    }
-#endif /* defined(HasDPS) */
   if (image == (Image *) NULL)
     if (UndefinedException == exception->severity)
       ThrowException(exception,DelegateError,PostscriptDelegateFailed,filename);
@@ -405,8 +403,8 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
     {
       do
         {
-          (void) strcpy(image->magick,"PS");
-          (void) strlcpy(image->filename,filename,MaxTextExtent);
+          (void) strlcpy(image->magick,"PS",sizeof(image->magick));
+          (void) strlcpy(image->filename,filename,sizeof(image->filename));
           next_image=SyncNextImageInList(image);
           if (next_image != (Image *) NULL)
             image=next_image;
@@ -416,6 +414,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
     }
   return(image);
 }
+#endif /* if defined(HasGS) */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -446,33 +445,39 @@ ModuleExport void RegisterEPTImage(void)
     *entry;
 
   entry=SetMagickInfo("EPT");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadEPTImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WriteEPTImage;
   entry->magick=(MagickHandler) IsEPT;
   entry->adjoin=False;
-  entry->blob_support=False;
+  entry->blob_support=MagickFalse;
   entry->description="Adobe Encapsulated PostScript with MS-DOS TIFF preview";
   entry->module="EPT";
   entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("EPT2");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadEPTImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WriteEPTImage;
   entry->magick=(MagickHandler) IsEPT;
   entry->adjoin=False;
-  entry->blob_support=False;
+  entry->blob_support=MagickFalse;
   entry->description="Adobe Level II Encapsulated PostScript with MS-DOS TIFF preview";
   entry->module="EPT";
   entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("EPT3");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadEPTImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WriteEPTImage;
   entry->magick=(MagickHandler) IsEPT;
   entry->adjoin=False;
-  entry->blob_support=False;
+  entry->blob_support=MagickFalse;
   entry->description="Adobe Level III Encapsulated PostScript with MS-DOS TIFF preview";
   entry->module="EPT";
   entry->coder_class=PrimaryCoderClass;
@@ -555,7 +560,7 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
 
-  logging=IsEventLogging();
+  logging=IsEventLogged(CoderEvent);
 
   (void) strlcpy(filename,image->filename,MaxTextExtent);
   (void) strlcpy(ps_filename,image->magick_filename,MaxTextExtent);
@@ -571,16 +576,16 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
         ThrowWriterTemporaryFileException(ps_filename);
 
       /* Select desired EPS level */
-      (void) strcpy(subformat,"eps");
+      (void) strlcpy(subformat,"eps",sizeof(subformat));
       if (LocaleCompare(image_info->magick,"EPT2") == 0)
-        (void) strcpy(subformat,"eps2");
+        (void) strlcpy(subformat,"eps2",sizeof(subformat));
       else if (LocaleCompare(image_info->magick,"EPT3") == 0)
-        (void) strcpy(subformat,"eps3");
+        (void) strlcpy(subformat,"eps3",sizeof(subformat));
 
       /* JPEG compression requires at least EPS2 */
       if ((image->compression == JPEGCompression) &&
           (LocaleCompare(subformat,"EPS") == 0))
-        (void) strcpy(subformat,"eps2");
+        (void) strlcpy(subformat,"eps2",sizeof(subformat));
 
       FormatString(image->filename,"%s:%.1024s",subformat,ps_filename);
       if (logging)
@@ -692,7 +697,7 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
                                 "Failed to open \"%s\" for read",ps_filename);
         }
 
-      CloseBlob(image);
+      status &= CloseBlob(image);
     }
   else
     {
@@ -705,5 +710,5 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
   (void) LiberateTemporaryFile(tiff_filename);
   if (status == MagickFail)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
-  return(MagickPass);
+  return(status);
 }
