@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2023 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -47,6 +47,7 @@
 #include "magick/pixel_cache.h"
 #include "magick/utility.h"
 #include "magick/version.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -62,16 +63,16 @@ static unsigned int
 /* Convert signed values to unsigned - and reverse. */
 static void FixSignedValues(unsigned char *data, int size, unsigned step, unsigned endian)
 {
-  if(endian != MSBEndian)
-  {
-    data += step - 1;  /* LSB has most signifficant byte at the end */
-  }                    /* MSB has most signifficant byte first */
+  if (endian != MSBEndian)
+    {
+      data += step - 1;  /* LSB has most signifficant byte at the end */
+    }                    /* MSB has most signifficant byte first */
 
-  while(size-->0)
-  {
-    *data ^= 0x80;
-    data += step;
-  }
+  while (size-->0)
+    {
+      *data ^= 0x80;
+      data += step;
+    }
 }
 
 /*
@@ -146,29 +147,29 @@ static unsigned int IsFITS(const unsigned char *magick,const size_t length)
 %
 %
 */
+typedef struct _FITSInfo
+{
+  char
+    extensions_exist;
+
+  int
+    simple,
+    bits_per_pixel,
+    columns,
+    rows,
+    number_axes,
+    number_scenes;
+
+  double
+    min_data,
+    max_data,
+    zero,
+    scale;
+} FITSInfo;
+
 static Image *ReadFITSImage(const ImageInfo *image_info,
                             ExceptionInfo *exception)
 {
-  typedef struct _FITSInfo
-  {
-    char
-      extensions_exist;
-
-    int
-      simple,
-      bits_per_pixel,
-      columns,
-      rows,
-      number_axes,
-      number_scenes;
-
-    double
-      min_data,
-      max_data,
-      zero,
-      scale;
-  } FITSInfo;
-
   char
     keyword[FITS_ROW_SIZE+1],
     value[FITS_ROW_SIZE+1];
@@ -201,7 +202,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     status,
     value_expected;
 
-  magick_uint64_t
+  magick_uintmax_t
     number_pixels;
 
   ImportPixelAreaOptions import_options;
@@ -369,9 +370,9 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
               if (ax_number > 0)
                 {
                   if (number_pixels == 0)
-                    number_pixels = (magick_uint64_t) y;
+                    number_pixels = (magick_uintmax_t) y;
                   else
-                    number_pixels *= (y <= 0) ? 1U : (magick_uint64_t) y;
+                    number_pixels *= (y <= 0) ? 1U : (magick_uintmax_t) y;
                 }
             }
 
@@ -409,7 +410,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     {
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                             "HDU read finished at offset %" MAGICK_OFF_F "Xh, "
-                            "number of pixels is: %" MAGICK_UINT64_F "u (%d*%d*%d)",
+                            "number of pixels is: %" MAGICK_UINTMAX_F "u (%d*%d*%d)",
                             TellBlob(image),
                             number_pixels,
                             fits_info.columns,
@@ -656,7 +657,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
                                             image->columns,image->rows))
                   break;
             }
-          MagickFreeResourceLimitedMemory(fits_pixels);
+          MagickFreeResourceLimitedMemory(unsigned char *,fits_pixels);
           if (EOFBlob(image))
             {
               ThrowException(exception,CorruptImageError,UnexpectedEndOfFile, image->filename);
@@ -688,8 +689,28 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 
   CloseBlob(image);
 
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  /*
+    Verify frames
+  */
+  image=GetFirstImageInList(image);
+  do
+    {
+      if ((image->columns == 0) || (image->rows == 0) || !GetPixelCachePresent(image))
+        {
+          if (logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Scene %lu: Image columns=%lu, rows=%lu, pixel-cache=%s",
+                                  image->scene, image->columns, image->rows,
+                                  GetPixelCachePresent(image) ? "Present" : "Missing!");
+          ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+        }
+      if (image->next)
+        image=image->next;
+      else
+        break;
+    } while (1);
+
+  image=GetFirstImageInList(image);
 
   if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),"return");
   return(image);
@@ -778,7 +799,7 @@ int InsertRowHDU(char *buffer, const char *data, int offset)
   len = Min(len,80); /* Each card image is 80 bytes max */
 
   if (len > (size_t) (((size_t)FITS_BLOCK_SIZE)-offset))
-        len = ((size_t) FITS_BLOCK_SIZE)-offset;
+    len = ((size_t) FITS_BLOCK_SIZE)-offset;
 
   (void) memcpy(buffer+offset,data,len);
   return offset +80;
@@ -861,115 +882,115 @@ static MagickPassFail WriteFITSImage(const ImageInfo *image_info,Image *image)
   export_options.sample_type = UnsignedQuantumSampleType;
 
   do
-  {
-    if (image->depth <= 8)
     {
-      quantum_size=8;
-    }
-    else if (image->depth <= 16)
-    {
-      quantum_size=16;
-    }
-    else
-    {
-      quantum_size=32;
-    }
-
-  /*
-    Allocate image memory.
-  */
-    packet_size=quantum_size/8;
-    fits_info=MagickAllocateResourceLimitedMemory(char *,FITS_BLOCK_SIZE);
-    if (fits_info == (char *) NULL)
-    {
-      ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
-    }
-    pixels=MagickAllocateResourceLimitedArray(unsigned char *,packet_size,image->columns);
-    if(pixels == (unsigned char *) NULL)
-    {
-      MagickFreeResourceLimitedMemory(fits_info);
-      ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
-    }
-
-  /*
-    Initialize image header.
-  */
-    memset(fits_info,' ',FITS_BLOCK_SIZE);
-    y = 0;
-    y = InsertRowHDU(fits_info, "SIMPLE  =                    T", y);
-   FormatString(buffer,        "BITPIX  =                    %u",quantum_size);
-    y = InsertRowHDU(fits_info, buffer, y);
-    y = InsertRowHDU(fits_info, "NAXIS   =                    2", y);
-    FormatString(buffer,        "NAXIS1  =           %10lu",image->columns);
-    y = InsertRowHDU(fits_info, buffer, y);
-    FormatString(buffer,        "NAXIS2  =           %10lu",image->rows);
-    y = InsertRowHDU(fits_info, buffer, y);
-    FormatString(buffer,        "DATAMIN =           %10u",0);
-    y = InsertRowHDU(fits_info, buffer, y);
-    FormatString(buffer,        "DATAMAX =           %10lu", MaxValueGivenBits(quantum_size));
-    y = InsertRowHDU(fits_info, buffer, y);
-   if (quantum_size>8)
-    {
-      FormatString(buffer,      "BZERO   =           %10u", (quantum_size <= 16) ? 32768U : 2147483648U);
-      y = InsertRowHDU(fits_info, buffer, y);
-    }
-  /* Magick version data can only be 60 bytes. */
-    (void) FormatString(buffer, "HISTORY Created by %.60s.",MagickPackageName " " MagickLibVersionText);
-    y = InsertRowHDU(fits_info, buffer, y);
-    if(image->next!=NULL)
-        y = InsertRowHDU(fits_info, "EXTEND   =                    T", y);
-    y = InsertRowHDU(fits_info, "END", y);
-    (void) WriteBlob(image, FITS_BLOCK_SIZE, (char *)fits_info);
-
-  /*
-    Convert image to fits scale PseudoColor class.
-  */
-    for (y=(long) image->rows-1; y >= 0; y--)
-    {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-      if (p == (const PixelPacket *) NULL)
-        break;
-      if (ExportImagePixelArea(image,GrayQuantum,quantum_size,pixels,&export_options,0) == MagickFail)
-        break;
-      if (quantum_size == 16)
-        FixSignedValues(pixels, image->columns, 2, export_options.endian);
-      if (quantum_size == 32)
-        FixSignedValues(pixels, image->columns, 4, export_options.endian);
-      if (WriteBlob(image, (size_t) packet_size*image->columns,pixels) != (size_t) packet_size*image->columns)
-        break;
-      if (QuantumTick((size_t) image->rows-y-1,image->rows))
+      if (image->depth <= 8)
         {
-          status=MagickMonitorFormatted((size_t) image->rows-y-1,image->rows,
-                                        &image->exception,SaveImageText,
-                                        image->filename,
-                                        image->columns,image->rows);
-          if (status == False)
-            break;
+          quantum_size=8;
+        }
+      else if (image->depth <= 16)
+        {
+          quantum_size=16;
+        }
+      else
+        {
+          quantum_size=32;
         }
-    }
 
-        /* Calculate of padding */
-    y = FITS_BLOCK_SIZE - (image->columns * image->rows * packet_size) % FITS_BLOCK_SIZE;
-    if(y > 0)
-      {
-        memset(fits_info, 0, y);
-        (void)WriteBlob(image,y,(char *) fits_info);
-      }
-    MagickFreeResourceLimitedMemory(fits_info);
-    MagickFreeResourceLimitedMemory(pixels);
+      /*
+        Allocate image memory.
+      */
+      packet_size=quantum_size/8;
+      fits_info=MagickAllocateResourceLimitedMemory(char *,FITS_BLOCK_SIZE);
+      if (fits_info == (char *) NULL)
+        {
+          ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+        }
+      pixels=MagickAllocateResourceLimitedArray(unsigned char *,packet_size,image->columns);
+      if (pixels == (unsigned char *) NULL)
+        {
+          MagickFreeResourceLimitedMemory(char *,fits_info);
+          ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+        }
 
-    if(image->next == (Image *) NULL)
-    {
-      if(image->logging)
+      /*
+        Initialize image header.
+      */
+      memset(fits_info,' ',FITS_BLOCK_SIZE);
+      y = 0;
+      y = InsertRowHDU(fits_info, "SIMPLE  =                    T", y);
+      FormatString(buffer,        "BITPIX  =                    %u",quantum_size);
+      y = InsertRowHDU(fits_info, buffer, y);
+      y = InsertRowHDU(fits_info, "NAXIS   =                    2", y);
+      FormatString(buffer,        "NAXIS1  =           %10lu",image->columns);
+      y = InsertRowHDU(fits_info, buffer, y);
+      FormatString(buffer,        "NAXIS2  =           %10lu",image->rows);
+      y = InsertRowHDU(fits_info, buffer, y);
+      FormatString(buffer,        "DATAMIN =           %10u",0);
+      y = InsertRowHDU(fits_info, buffer, y);
+      FormatString(buffer,        "DATAMAX =           %10lu", MaxValueGivenBits(quantum_size));
+      y = InsertRowHDU(fits_info, buffer, y);
+      if (quantum_size>8)
+        {
+          FormatString(buffer,      "BZERO   =           %10u", (quantum_size <= 16) ? 32768U : 2147483648U);
+          y = InsertRowHDU(fits_info, buffer, y);
+        }
+      /* Magick version data can only be 60 bytes. */
+      (void) FormatString(buffer, "HISTORY Created by %.60s.",MagickPackageName " " MagickLibVersionText);
+      y = InsertRowHDU(fits_info, buffer, y);
+      if (image->next!=NULL)
+        y = InsertRowHDU(fits_info, "EXTEND   =                    T", y);
+      y = InsertRowHDU(fits_info, "END", y);
+      (void) WriteBlob(image, FITS_BLOCK_SIZE, (char *)fits_info);
+
+      /*
+        Convert image to fits scale PseudoColor class.
+      */
+      for (y=(long) image->rows-1; y >= 0; y--)
+        {
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
+            break;
+          if (ExportImagePixelArea(image,GrayQuantum,quantum_size,pixels,&export_options,0) == MagickFail)
+            break;
+          if (quantum_size == 16)
+            FixSignedValues(pixels, image->columns, 2, export_options.endian);
+          if (quantum_size == 32)
+            FixSignedValues(pixels, image->columns, 4, export_options.endian);
+          if (WriteBlob(image, (size_t) packet_size*image->columns,pixels) != (size_t) packet_size*image->columns)
+            break;
+          if (QuantumTick((size_t) image->rows-y-1,image->rows))
+            {
+              status=MagickMonitorFormatted((size_t) image->rows-y-1,image->rows,
+                                            &image->exception,SaveImageText,
+                                            image->filename,
+                                            image->columns,image->rows);
+              if (status == False)
+                break;
+            }
+        }
+
+      /* Calculate of padding */
+      y = FITS_BLOCK_SIZE - (image->columns * image->rows * packet_size) % FITS_BLOCK_SIZE;
+      if (y > 0)
+        {
+          memset(fits_info, 0, y);
+          (void)WriteBlob(image,y,(char *) fits_info);
+        }
+      MagickFreeResourceLimitedMemory(char *,fits_info);
+      MagickFreeResourceLimitedMemory(unsigned char *,pixels);
+
+      if (image->next == (Image *) NULL)
+        {
+          if (image->logging)
             (void)LogMagickEvent(CoderEvent,GetMagickModule(), "No more image frames in list.");
-      break;
-    }
-    image = SyncNextImageInList(image);
-  } while(1);
+          break;
+        }
+      image = SyncNextImageInList(image);
+    } while (1);
 
-        /* Rewind image list. */
+  /* Rewind image list. */
   while (image->previous != (Image *)NULL)
-      image=image->previous;
+    image=image->previous;
 
   status &= CloseBlob(image);
   return(status);
