@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2023 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -45,6 +45,7 @@
 #include "magick/pixel_cache.h"
 #include "magick/resource.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -55,7 +56,13 @@ static unsigned int
 #endif
 
 #if defined(HasJBIG)
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
 #include "jbig.h"
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -221,6 +228,9 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   if ((image_info->type != GrayscaleType) &&
       (image_info->type != TrueColorType))
     {
+      if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Allocating colormap...");
       if (!AllocateImageColormap(image,2))
         {
           MagickFreeMemory(buffer);
@@ -229,9 +239,11 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
       image->colormap[0].red=0;
       image->colormap[0].green=0;
       image->colormap[0].blue=0;
+      image->colormap[0].opacity=0;
       image->colormap[1].red=MaxRGB;
       image->colormap[1].green=MaxRGB;
       image->colormap[1].blue=MaxRGB;
+      image->colormap[1].opacity=0;
     }
   image->x_resolution=300;
   image->y_resolution=300;
@@ -528,7 +540,9 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
     jbg_enc_init(&jbig_info,image->columns,image->rows,1,&pixels,
       (void (*)(unsigned char *,size_t,void *)) JBIGEncode,image);
     if (image_info->subimage != 0)
-      jbg_enc_layers(&jbig_info,(int) image_info->subimage);
+      {
+        jbg_enc_layers(&jbig_info,(int) image_info->subimage);
+      }
     else
       {
         long
@@ -559,7 +573,7 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
     */
     jbg_enc_out(&jbig_info);
     jbg_enc_free(&jbig_info);
-    MagickFreeResourceLimitedMemory(pixels);
+    MagickFreeResourceLimitedMemory(unsigned char *,pixels);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
