@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2012-2018 GraphicsMagick Group
+% Copyright (C) 2012-2025 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -45,6 +45,7 @@
 #include "magick/magic.h"
 #include "magick/resource.h"
 #include "magick/attribute.h"
+#include "magick/static.h"
 
 typedef struct
 {
@@ -101,7 +102,7 @@ ExtractTileJPG(Image * image, const ImageInfo * image_info,
   char img_label_str[MaxTextExtent];
 
 
-  alloc_size = TileInfo->PicSize + 2;
+  alloc_size = (size_t) TileInfo->PicSize + 2;
 
   if (image->logging)
     (void) LogMagickEvent(CoderEvent, GetMagickModule(),
@@ -109,7 +110,7 @@ ExtractTileJPG(Image * image, const ImageInfo * image_info,
                           TileInfo->PicOffset, TileInfo->PicSize);
 
   if ((alloc_size > TileInfo->PicSize) &&
-      (blob = MagickAllocateMemory(unsigned char *,alloc_size)) != NULL)
+      (blob = MagickAllocateResourceLimitedMemory(unsigned char *,alloc_size)) != NULL)
     {
       /* Add missing JPEG header bytes */
       blob[0] = 0xFF;
@@ -123,7 +124,15 @@ ExtractTileJPG(Image * image, const ImageInfo * image_info,
               Image
                 *image2;
 
-              if ((image2 = BlobToImage(image_info,blob,alloc_size,exception))
+              ImageInfo
+                *clone_info;
+
+              clone_info=CloneImageInfo(image_info);
+
+              /* BlobToFile("/tmp/jnx-tile.jpg", blob,alloc_size,exception); */
+
+              (void) strlcpy(clone_info->filename,"JPEG:",sizeof(clone_info->filename));
+              if ((image2 = BlobToImage(clone_info,blob,alloc_size,exception))
                   != NULL)
                 {
                   /*
@@ -143,17 +152,19 @@ ExtractTileJPG(Image * image, const ImageInfo * image_info,
                     DeleteImageFromList(&image);
 
                   FormatString(img_label_str, "%.20g;%.20g",
-                    (double) TileInfo->TileBounds.NorthEast.lat*180.0/0x7FFFFFFF,
-                    (double) TileInfo->TileBounds.NorthEast.lon*180.0/0x7FFFFFFF);
+                               (double) TileInfo->TileBounds.NorthEast.lat*180.0/0x7FFFFFFF,
+                               (double) TileInfo->TileBounds.NorthEast.lon*180.0/0x7FFFFFFF);
                   SetImageAttribute(image2, "jnx:northeast", img_label_str);
 
                   FormatString(img_label_str, "%.20g;%.20g",
-                    (double) TileInfo->TileBounds.SouthWest.lat*180.0/0x7FFFFFFF,
-                    (double) TileInfo->TileBounds.SouthWest.lon*180.0/0x7FFFFFFF);
+                               (double) TileInfo->TileBounds.SouthWest.lat*180.0/0x7FFFFFFF,
+                               (double) TileInfo->TileBounds.SouthWest.lon*180.0/0x7FFFFFFF);
                   SetImageAttribute(image2, "jnx:southwest", img_label_str);
 
                   AppendImageToList(&image, image2);
                 }
+              DestroyImageInfo(clone_info);
+              clone_info = (ImageInfo *) NULL;
             }
           else
             {
@@ -168,14 +179,14 @@ ExtractTileJPG(Image * image, const ImageInfo * image_info,
           ThrowException(exception,BlobError,UnableToSeekToOffset,
                          image->filename);
         }
-      MagickFreeMemory(blob);
+      MagickFreeResourceLimitedMemory(unsigned char *,blob);
     }
   else
     {
       /* Failed to allocate memory */
       ThrowException(exception,ResourceLimitError,MemoryAllocationFailed,
                      image->filename);
-    }  
+    }
 
   return(image);
 }
@@ -253,6 +264,8 @@ ReadJNXImage(const ImageInfo * image_info, ExceptionInfo * exception)
   logging = LogMagickEvent(CoderEvent, GetMagickModule(), "enter");
 
   image = AllocateImage(image_info);
+  image->rows = 0;
+  image->columns = 0;
   status = OpenBlob(image_info, image, ReadBinaryBlobMode, exception);
   if (status == False)
     ThrowReaderException(FileOpenError, UnableToOpenFile, image);
@@ -379,7 +392,7 @@ ReadJNXImage(const ImageInfo * image_info, ExceptionInfo * exception)
           ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
         }
 
-      PositionList = MagickAllocateArray(TJNXTileInfo *,
+      PositionList = MagickAllocateResourceLimitedArray(TJNXTileInfo *,
                                          JNXLevelInfo[i].TileCount,
                                          sizeof(TJNXTileInfo));
       if (PositionList == NULL)
@@ -406,7 +419,7 @@ ReadJNXImage(const ImageInfo * image_info, ExceptionInfo * exception)
                PositionList[j].PicSize > file_size))
             {
               (void) SetMagickResourceLimit(MapResource, SaveLimit);
-              MagickFreeMemory(PositionList);
+              MagickFreeResourceLimitedMemory(TJNXTileInfo *,PositionList);
               ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
             }
         }
@@ -432,7 +445,7 @@ ReadJNXImage(const ImageInfo * image_info, ExceptionInfo * exception)
               break;
         }
 
-      MagickFreeMemory(PositionList);
+      MagickFreeResourceLimitedMemory(TJNXTileInfo *,PositionList);
     }
 
   CloseBlob(image);
