@@ -46,6 +46,7 @@
 #include "magick/profile.h"
 #include "magick/resource.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 #if defined(HasJP2)
 #  if !defined(uchar)
 #    define uchar  unsigned char
@@ -455,7 +456,7 @@ static jas_stream_t *JP2StreamManager(jas_stream_ops_t *stream_ops, Image *image
 #define ThrowJP2ReaderException(code_,reason_,image_)                   \
   {                                                                     \
     for (component=0; component < (long) number_components; component++) \
-      MagickFreeResourceLimitedMemory(channel_lut[component]);          \
+      MagickFreeResourceLimitedMemory(Quantum *,channel_lut[component]); \
     if (pixels)                                                         \
       jas_matrix_destroy(pixels);                                       \
     if (jp2_stream)                                                     \
@@ -482,7 +483,7 @@ static void *alloc_rlm(struct jas_allocator_s *allocator, size_t size)
   char *p;
   (void) allocator;
   /* JasPer expects its allocator to return non-null for zero size */
-  p=_MagickAllocateResourceLimitedMemoryLoc((size == 0 ? 1 : size),GetMagickModule());
+  p=(char *) _MagickAllocateResourceLimitedMemoryLoc((size == 0 ? 1 : size),GetMagickModule());
   /* fprintf(stderr,"alloc_rlm(%p, %zu) -> %p\n", allocator, size, p); */
   return p;
 }
@@ -497,7 +498,7 @@ static void *realloc_rlm(struct jas_allocator_s *allocator, void *pointer,
 {
   char *p;
   (void) allocator;
-  p =_MagickReallocateResourceLimitedMemoryLoc(pointer, 1,new_size,0,GetMagickModule());
+  p =(char *) _MagickReallocateResourceLimitedMemoryLoc(pointer, 1,new_size,0,GetMagickModule());
   /* fprintf(stderr,"realloc_rlm(%p, %p, %zu) -> %p\n", allocator, pointer, new_size, p); */
   return p;
 }
@@ -1211,7 +1212,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,
   }
 
   for (component=0; component < (long) number_components; component++)
-    MagickFreeResourceLimitedMemory(channel_lut[component]);
+    MagickFreeResourceLimitedMemory(Quantum *,channel_lut[component]);
   jas_matrix_destroy(pixels);
   (void) jas_stream_close(jp2_stream);
   MagickFreeMemory(options);
@@ -1654,7 +1655,7 @@ WriteJP2Image(const ImageInfo *image_info,Image *image)
   jp2_pixels=jas_matrix_create(1,(unsigned int) image->columns);
   if (jp2_pixels == (jas_matrix_t *) NULL)
     {
-      MagickFreeResourceLimitedMemory(lut);
+      MagickFreeResourceLimitedMemory(unsigned short *,lut);
       jas_image_destroy(jp2_image);
       ThrowJP2WriterException(ResourceLimitError,MemoryAllocationFailed,image);
     }
@@ -1759,7 +1760,7 @@ WriteJP2Image(const ImageInfo *image_info,Image *image)
   status=jas_image_encode(jp2_image,jp2_stream,format,options);
   (void) jas_stream_close(jp2_stream);
   MagickFreeMemory(options);
-  MagickFreeResourceLimitedMemory(lut);
+  MagickFreeResourceLimitedMemory(unsigned short *,lut);
   jas_matrix_destroy(jp2_pixels);
   jas_image_destroy(jp2_image);
   if (status)
