@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2020 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -41,12 +41,14 @@
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
 */
 static unsigned int
-  WriteLOCALEImage(const ImageInfo *,Image *);
+WriteLOCALEImage(const ImageInfo *,Image *);
+
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -217,7 +219,7 @@ static unsigned int ReadConfigureFile(Image *image,const char *basename,
     if (LocaleCompare(keyword,"</locale>") == 0)
       {
         ChopLocaleComponents(locale,1);
-        (void) strcat(locale,"/");
+        (void) strlcat(locale,"/",sizeof(locale));
         continue;
       }
     if (LocaleCompare(keyword,"<localemap>") == 0)
@@ -247,7 +249,7 @@ static unsigned int ReadConfigureFile(Image *image,const char *basename,
         {
           (void) strncpy(message,p,(size_t)(q-p));
           message[q-p]='\0';
-          Strip(message);
+          (void) MagickStripString(message);
           (void) strlcat(locale,message,sizeof(locale));
           (void) strlcat(locale,"\n",sizeof(locale));
           (void) SetImageAttribute(image,"[Locale]",locale);
@@ -257,7 +259,7 @@ static unsigned int ReadConfigureFile(Image *image,const char *basename,
     if (LocaleCompare(keyword,"</message>") == 0)
       {
         ChopLocaleComponents(locale,2);
-        (void) strcat(locale,"/");
+        (void) strlcat(locale,"/",sizeof(locale));
         continue;
       }
     if (*keyword == '<')
@@ -270,7 +272,7 @@ static unsigned int ReadConfigureFile(Image *image,const char *basename,
         if (*(keyword+1) == '/')
           {
             ChopLocaleComponents(locale,1);
-            (void) strcat(locale,"/");
+            (void) strlcat(locale,"/",sizeof(locale));
             continue;
           }
         token[strlen(token)-1]='\0';
@@ -378,7 +380,7 @@ ModuleExport void RegisterLOCALEImage(void)
   entry->decoder=(DecoderHandler) ReadLOCALEImage;
   entry->encoder=(EncoderHandler) WriteLOCALEImage;
   entry->adjoin=False;
-  entry->stealth=True;
+  entry->stealth=True; /* Don't list in '-list format' output */
   entry->description="Locale Message File";
   entry->module="LOCALE";
   (void) RegisterMagickInfo(entry);
@@ -386,7 +388,7 @@ ModuleExport void RegisterLOCALEImage(void)
   entry=SetMagickInfo("LOCALEMC");
   entry->encoder=(EncoderHandler) WriteLOCALEImage;
   entry->adjoin=False;
-  entry->stealth=True;
+  entry->stealth=True; /* Don't list in '-list format' output */
   entry->description="Microsoft Message File";
   entry->module="LOCALE";
   (void) RegisterMagickInfo(entry);
@@ -394,7 +396,7 @@ ModuleExport void RegisterLOCALEImage(void)
   entry=SetMagickInfo("LOCALEC");
   entry->encoder=(EncoderHandler) WriteLOCALEImage;
   entry->adjoin=False;
-  entry->stealth=True;
+  entry->stealth=True; /* Don't list in '-list format' output */
   entry->description="Locale Message File - C code";
   entry->module="LOCALE";
   (void) RegisterMagickInfo(entry);
@@ -402,7 +404,7 @@ ModuleExport void RegisterLOCALEImage(void)
   entry=SetMagickInfo("LOCALEH");
   entry->encoder=(EncoderHandler) WriteLOCALEImage;
   entry->adjoin=False;
-  entry->stealth=True;
+  entry->stealth=True; /* Don't list in '-list format' output */
   entry->description="Locale Message File - C header file";
   entry->module="LOCALE";
   (void) RegisterMagickInfo(entry);
@@ -807,7 +809,7 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
           locale[i]=locale[j];
           locale[j]=swap;
         }
-  if (IsEventLogging())
+  if (IsEventLogged(LocaleEvent))
     for (i=0; i < count; i++)
       (void) LogMagickEvent(LocaleEvent,GetMagickModule(),"%.1024s",locale[i]);
   if (LocaleCompare(image_info->magick,"LOCALEMC") == 0)
@@ -820,20 +822,24 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
         char
           *fields[4],
           text[MaxTextExtent],
-          path[MaxTextExtent];
+          path[MaxTextExtent],
+          emptystr[1];
 
-        int
+        unsigned int
           index;
 
         register char
           *p;
 
         (void) strlcpy(path,locale[i],sizeof(path));
+        emptystr[0]='\0';
         if (*path != '\0')
           {
             p=path+strlen(path)-1;
             if (*p == '/')
               *p='\0';
+            for (index=0; index < ArraySize(fields) ; index++)
+              fields[index]=emptystr;
             for (index=0; (index < 4) && (p > path); p--)
             {
               if (*p == '/')
@@ -873,8 +879,10 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
         text[MaxTextExtent],
         path[MaxTextExtent];
 
+      unsigned int
+        index;
+
       int
-        index,
         severityindex;
 
       size_t
@@ -902,7 +910,8 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
             p=path+strlen(path)-1;
             if (*p == '/')
               *p='\0';
-            fields[0]=""; /* this one may not exist */
+            for (index=0; index < ArraySize(fields) ; index++)
+              fields[index]="";
             for (index=0; (index < TREE_LEVELS_SUPPORTED) && (p > path); p--)
             {
               if (*p == '/')
@@ -942,7 +951,8 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
             p=path+strlen(path)-1;
             if (*p == '/')
               *p='\0';
-            fields[0]=""; /* this one may not exist */
+            for (index=0; index < ArraySize(fields) ; index++)
+              fields[index]="";
             for (index=0; (index < TREE_LEVELS_SUPPORTED) && (p > path); p--)
             {
               if (*p == '/')
@@ -994,7 +1004,8 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
             p=path+strlen(path)-1;
             if (*p == '/')
               *p='\0';
-            fields[0]=""; /* this one may not exist */
+            for (index=0; index < ArraySize(fields) ; index++)
+              fields[index]="";
             for (index=0; (index < TREE_LEVELS_SUPPORTED) && (p > path); p--)
             {
               if (*p == '/')
@@ -1040,7 +1051,8 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
             p=path+strlen(path)-1;
             if (*p == '/')
               *p='\0';
-            fields[0]=""; /* this one may not exist */
+            for (index=0; index < ArraySize(fields) ; index++)
+              fields[index]="";
             for (index=0; (index < TREE_LEVELS_SUPPORTED) && (p > path); p--)
             {
               if (*p == '/')
@@ -1075,7 +1087,8 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
             p=path+strlen(path)-1;
             if (*p == '/')
               *p='\0';
-            fields[0]=""; /* this one may not exist */
+            for (index=0; index < ArraySize(fields) ; index++)
+              fields[index]="";
             for (index=0; (index < TREE_LEVELS_SUPPORTED) && (p > path); p--)
             {
               if (*p == '/')
@@ -1112,7 +1125,8 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
             p=path+strlen(path)-1;
             if (*p == '/')
               *p='\0';
-            fields[0]=""; /* this one may not exist */
+            for (index=0; index < ArraySize(fields) ; index++)
+              fields[index]="";
             for (index=0; (index < TREE_LEVELS_SUPPORTED) && (p > path); p--)
             {
               if (*p == '/')
@@ -1156,6 +1170,6 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
   for (i=0; i <= count; i++)
     MagickFreeMemory(locale[i]);
   MagickFreeMemory(locale);
-  CloseBlob(image);
-  return(True);
+  status &= CloseBlob(image);
+  return(status);
 }
