@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2018 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -40,6 +40,7 @@
 #include "magick/magick.h"
 #include "magick/profile.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -127,18 +128,15 @@ static unsigned int IsMETA(const unsigned char *magick,const size_t length)
 */
 #define BUFFER_SZ 4096
 
-typedef struct _html_code
+static const struct
 {
-  unsigned short
+  const unsigned char
     len;
 
   const char
-    *code,
+    code[7],
     val;
-
-} html_code;
-
-static html_code const html_codes[] = {
+} html_codes[] = {
 #ifdef HANDLE_GT_LT
   { 4,"&lt;",'<' },
   { 4,"&gt;",'>' },
@@ -181,10 +179,29 @@ static int stringnicmp(const char *p,const char *q,size_t n)
   return(toupper((int) *p)-toupper((int) *q));
 }
 
-static size_t convertHTMLcodes(char *s, const size_t len)
+/*
+   This routine parses the next HTML ampersand code ("&FOO;") from the
+   provided string and replaces the HTML ampersand code with the
+   equivalent character, moving the remainder of the string forward.
+   The return value is the number of characters consumed (length of
+   code minus one character).
+ */
+static size_t convertHTMLcodes(char *s)
 {
+  size_t len;
+  size_t i;
   int val;
 
+  for (i = 0, len = 0; (i < 7U) && (s[i] != '\0') ; i++)
+    {
+      if (s[i] == ';')
+        {
+          len=i+1;
+          break;
+        }
+    }
+
+
   if ((len == 0) || (s == (char*) NULL) || (*s=='\0'))
     return 0;
 
@@ -197,8 +214,7 @@ static size_t convertHTMLcodes(char *s, const size_t len)
     and that it does not require the terminating ';' to appear in
     order to parse the numeric argument.
   */
-  if ((len > 3) && (s[1] == '#') && (strchr(s,';') != (char *) NULL) &&
-      (sscanf(s,"&#%d;",&val) == 1))
+  if ((len > 3) && (s[1] == '#') && (sscanf(s,"&#%d;",&val) == 1))
     {
       size_t o = 3;
       while (s[o] != ';')
@@ -212,11 +228,11 @@ static size_t convertHTMLcodes(char *s, const size_t len)
       *s = val;
       return o;
     }
-  else
+  else /* Symbolic code, e.g. "&amp;" */
     {
       int
         i,
-        codes = sizeof(html_codes) / sizeof(html_code);
+        codes = ArraySize(html_codes);
 
       for (i=0; i < codes; i++)
       {
@@ -251,34 +267,41 @@ static char *super_fgets(char **b, size_t *blen, Image *file)
     c=ReadBlobByte(file);
     if (c == EOF || c == '\n')
       break;
-    if ((q-p+1) >= (int) len)
+    if ((size_t) (q-p+1) >= len)
       {
-        int
+        unsigned char
+          *new_p;
+
+        size_t
           tlen;
 
-        tlen=q-p;
-        len<<=1;
-        MagickReallocMemory(unsigned char *,p,(len+2));
-        *b=(char *) p;
-        if (p == (unsigned char *) NULL)
-          break;
+        tlen=(size_t) (q-p);
+        len += MaxTextExtent;
+        new_p=MagickReallocateResourceLimitedMemory(unsigned char *,p,(len+2));
+        if (new_p == (unsigned char *) NULL)
+          {
+            MagickFreeResourceLimitedMemory(unsigned char *,p);
+            break;
+          }
+        p=new_p;
         q=p+tlen;
       }
     *q=(unsigned char) c;
   }
+  *b=(char *)p;
   *blen=0;
   if (p != (unsigned char *) NULL)
     {
-      int
+      size_t
         tlen;
 
-      tlen=q-p;
+      tlen=(size_t) (q-p);
       if (tlen == 0)
         return (char *) NULL;
       p[tlen] = '\0';
       *blen=++tlen;
     }
-  return (char *)p;
+  return *b;
 }
 
 #define BUFFER_SZ 4096
@@ -321,7 +344,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
 
   dataset = 0;
   recnum = 0;
-  line = MagickAllocateMemory(char *,inputlen);
+  line = MagickAllocateResourceLimitedMemory(char *,inputlen);
   if (line == (char *) NULL)
     goto parse8BIM_failure;
   savedpos = 0;
@@ -330,10 +353,10 @@ static long parse8BIM(Image *ifile, Image *ofile)
     state=0;
     next=0;
 
-    token = MagickAllocateMemory(char *,inputlen);
+    token = MagickAllocateResourceLimitedMemory(char *,inputlen);
     if (token == (char *) NULL)
       goto parse8BIM_failure;
-    newstr = MagickAllocateMemory(char *,inputlen);
+    newstr = MagickAllocateResourceLimitedMemory(char *,inputlen);
     if (newstr == (char *) NULL)
       goto parse8BIM_failure;
     while (Tokenizer(&token_info, 0, token, inputlen, line,
@@ -342,6 +365,9 @@ static long parse8BIM(Image *ifile, Image *ofile)
     {
       if (state == 0)
         {
+          size_t
+            name_alloc;
+
           int
             state,
             next;
@@ -367,10 +393,11 @@ static long parse8BIM(Image *ifile, Image *ofile)
                 recnum = MagickAtoI(newstr);
                 break;
               case 2:
-                name = MagickAllocateMemory(char *,strlen(newstr)+1);
+                name_alloc = strlen(newstr)+1;
+                name = MagickAllocateResourceLimitedMemory(char *,name_alloc);
                 if (name == (char *) NULL)
                   goto parse8BIM_failure;
-                (void) strcpy(name,newstr);
+                (void) strlcpy(name,newstr,name_alloc);
                 break;
             }
             state++;
@@ -402,7 +429,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
                   char
                     *s = &token[next-1];
 
-                  codes_len = convertHTMLcodes(s, strlen(s));
+                  codes_len = convertHTMLcodes(s);
                   if (codes_len > len)
                     len = 0;
                   else
@@ -424,44 +451,56 @@ static long parse8BIM(Image *ifile, Image *ofile)
                     currentpos = TellBlob(ofile);
                     if (currentpos < 0)
                       goto parse8BIM_failure;
-                    (void) SeekBlob(ofile,savedpos,SEEK_SET);
-                    (void) WriteBlobMSBLong(ofile,diff);
-                    (void) SeekBlob(ofile,currentpos,SEEK_SET);
+                    if (SeekBlob(ofile,savedpos,SEEK_SET) != savedpos)
+                      goto parse8BIM_failure;
+                    if (WriteBlobMSBLong(ofile,diff) < 4)
+                      goto parse8BIM_failure;
+                    if (SeekBlob(ofile,currentpos,SEEK_SET) != currentpos)
+                      goto parse8BIM_failure;
                     savedolen = 0L;
                   }
                 if (outputlen & 1)
                   {
-                    (void) WriteBlobByte(ofile,0x00);
+                    if (WriteBlobByte(ofile,0x00) < 1)
+                      goto parse8BIM_failure;
                     outputlen++;
                   }
-                (void) WriteBlobString(ofile,"8BIM");
-                (void) WriteBlobMSBShort(ofile,recnum);
+                if (WriteBlobString(ofile,"8BIM") < 4)
+                  goto parse8BIM_failure;
+                if (WriteBlobMSBShort(ofile,recnum) < 2)
+                  goto parse8BIM_failure;
                 outputlen += 6;
                 if (name)
                   nlen = (unsigned char) strlen(name);
-                (void) WriteBlobByte(ofile,nlen);
+                if (WriteBlobByte(ofile,nlen) < 1)
+                  goto parse8BIM_failure;
                 outputlen++;
                 for (i=0; i<nlen; i++)
-                  (void) WriteBlobByte(ofile,name[i]);
+                  if (WriteBlobByte(ofile,name[i]) < 1)
+                    goto parse8BIM_failure;
                 outputlen += nlen;
                 if (!(nlen&1))
                   {
-                    (void) WriteBlobByte(ofile,0x00);
+                    if (WriteBlobByte(ofile,0x00) < 1)
+                      goto parse8BIM_failure;
                     outputlen++;
                   }
                 if (recnum != IPTC_ID)
                   {
-                    (void) WriteBlobMSBLong(ofile, (const magick_uint32_t) len);
+                    if (WriteBlobMSBLong(ofile, (magick_uint32_t) len) < 4)
+                      goto parse8BIM_failure;
                     outputlen += 4;
 
                     next=0;
                     outputlen += len;
                     while (len--)
-                      (void) WriteBlobByte(ofile,token[next++]);
+                      if (WriteBlobByte(ofile,token[next++]) < 1)
+                        goto parse8BIM_failure;
 
                     if (outputlen & 1)
                       {
-                        (void) WriteBlobByte(ofile,0x00);
+                        if (WriteBlobByte(ofile,0x00) < 1)
+                          goto parse8BIM_failure;
                         outputlen++;
                       }
                   }
@@ -471,7 +510,8 @@ static long parse8BIM(Image *ifile, Image *ofile)
                     savedpos = TellBlob(ofile);
                     if (savedpos < 0)
                       goto parse8BIM_failure;
-                    (void) WriteBlobMSBLong(ofile,0xFFFFFFFFUL);
+                    if (WriteBlobMSBLong(ofile,0xFFFFFFFFUL) < 4)
+                      goto parse8BIM_failure;
                     outputlen += 4;
                     savedolen = outputlen;
                   }
@@ -480,25 +520,30 @@ static long parse8BIM(Image *ifile, Image *ofile)
               {
                 if (len <= 0x7FFF)
                   {
-                    (void) WriteBlobByte(ofile,0x1c);
-                    (void) WriteBlobByte(ofile,dataset);
-                    (void) WriteBlobByte(ofile,recnum & 255);
-                    (void) WriteBlobMSBShort(ofile,(const magick_uint16_t) len);
+                    if (WriteBlobByte(ofile,0x1c) < 1)
+                      goto parse8BIM_failure;
+                    if (WriteBlobByte(ofile,dataset) < 1)
+                      goto parse8BIM_failure;
+                    if (WriteBlobByte(ofile,recnum & 255) < 1)
+                      goto parse8BIM_failure;
+                    if (WriteBlobMSBShort(ofile,(magick_uint16_t) len) < 2)
+                      goto parse8BIM_failure;
                     outputlen += 5;
                     next=0;
                     outputlen += len;
                     while (len--)
-                      (void) WriteBlobByte(ofile,token[next++]);
+                      if (WriteBlobByte(ofile,token[next++]) < 1)
+                        goto parse8BIM_failure;
                   }
               }
           }
       state++;
     }
-    MagickFreeMemory(token);
-    MagickFreeMemory(newstr);
-    MagickFreeMemory(name);
+    MagickFreeResourceLimitedMemory(char *,token);
+    MagickFreeResourceLimitedMemory(char *,newstr);
+    MagickFreeResourceLimitedMemory(char *,name);
   }
-  MagickFreeMemory(line);
+  MagickFreeResourceLimitedMemory(char *,line);
   if (savedolen > 0)
     {
       long diff = outputlen - savedolen;
@@ -507,18 +552,21 @@ static long parse8BIM(Image *ifile, Image *ofile)
       if (currentpos < 0)
         goto parse8BIM_failure;
 
-      (void) SeekBlob(ofile,savedpos,SEEK_SET);
-      (void) WriteBlobMSBLong(ofile,diff);
-      (void) SeekBlob(ofile,currentpos,SEEK_SET);
+      if (SeekBlob(ofile,savedpos,SEEK_SET) != savedpos)
+        goto parse8BIM_failure;
+      if (WriteBlobMSBLong(ofile,diff) < 4)
+        goto parse8BIM_failure;
+      if (SeekBlob(ofile,currentpos,SEEK_SET) != currentpos)
+        goto parse8BIM_failure;
       savedolen = 0L;
     }
   return outputlen;
 
  parse8BIM_failure:
-  MagickFreeMemory(token);
-  MagickFreeMemory(newstr);
-  MagickFreeMemory(name);
-  MagickFreeMemory(line);
+  MagickFreeResourceLimitedMemory(char *,token);
+  MagickFreeResourceLimitedMemory(char *,newstr);
+  MagickFreeResourceLimitedMemory(char *,name);
+  MagickFreeResourceLimitedMemory(char *,line);
   return 0L;
 }
 
@@ -543,34 +591,41 @@ static char *super_fgets_w(char **b, size_t *blen, Image *file)
       break;
    if (EOFBlob(file))
       break;
-   if ((q-p+1) >= (int) len)
+   if ((size_t) (q-p+1) >= len)
       {
-        int
+        unsigned char
+          *new_p;
+
+        size_t
           tlen;
 
-        tlen=q-p;
-        len<<=1;
-        MagickReallocMemory(unsigned char *,p,(len+2));
-        *b=(char *) p;
-        if (p == (unsigned char *) NULL)
-          break;
+        tlen=(size_t) (q-p);
+        len += MaxTextExtent;
+        new_p=MagickReallocateResourceLimitedMemory(unsigned char *,p,(len+2));
+        if (new_p == (unsigned char *) NULL)
+          {
+            MagickFreeResourceLimitedMemory(unsigned char *,p);
+            break;
+          }
+        p=new_p;
         q=p+tlen;
       }
     *q=(unsigned char) c;
   }
+  *b=(char *)p;
   *blen=0;
-  if ((*b) != (char *) NULL)
+  if (p != (unsigned char *) NULL)
     {
-      int
+      size_t
         tlen;
 
-      tlen=q-p;
+      tlen=(size_t) (q-p);
       if (tlen == 0)
         return (char *) NULL;
       p[tlen] = '\0';
       *blen=++tlen;
     }
-  return (char *) p;
+  return *b;
 }
 
 static long parse8BIMW(Image *ifile, Image *ofile)
@@ -609,7 +664,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
 
   dataset = 0;
   recnum = 0;
-  line = MagickAllocateMemory(char *,inputlen);
+  line = MagickAllocateResourceLimitedMemory(char *,inputlen);
   if (line == (char *) NULL)
     goto parse8BIMW_failure;
   name = token = (char *)NULL;
@@ -623,10 +678,10 @@ static long parse8BIMW(Image *ifile, Image *ofile)
 
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                           "META CODER Parse8BIM: %s (%lu)",line, (unsigned long) inputlen);
-    token = MagickAllocateMemory(char *,inputlen);
+    token = MagickAllocateResourceLimitedMemory(char *,inputlen);
     if (token == (char *) NULL)
       goto parse8BIMW_failure;
-    newstr = MagickAllocateMemory(char *,inputlen);
+    newstr = MagickAllocateResourceLimitedMemory(char *,inputlen);
     if (newstr == (char *) NULL)
       goto parse8BIMW_failure;
     while (Tokenizer(&token_info, 0, token, inputlen, line,
@@ -635,6 +690,9 @@ static long parse8BIMW(Image *ifile, Image *ofile)
     {
       if (state == 0)
         {
+          size_t
+            name_alloc;
+
           int
             state,
             next;
@@ -660,10 +718,11 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                 recnum = MagickAtoI(newstr);
                 break;
               case 2:
-                name = MagickAllocateMemory(char *,strlen(newstr)+1);
+                name_alloc = strlen(newstr)+1;
+                name = MagickAllocateResourceLimitedMemory(char *,name_alloc);
                 if (name == (char *) NULL)
                   goto parse8BIMW_failure;
-                (void) strcpy(name,newstr);
+                (void) strlcpy(name,newstr,name_alloc);
                 break;
             }
             state++;
@@ -695,7 +754,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                   char
                     *s = &token[next-1];
 
-                  codes_len = convertHTMLcodes(s, strlen(s));
+                  codes_len = convertHTMLcodes(s);
                   if (codes_len > len)
                     len = 0;
                   else
@@ -717,44 +776,56 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                     currentpos = TellBlob(ofile);
                     if (currentpos < 0)
                       goto parse8BIMW_failure;
-                    (void) SeekBlob(ofile,savedpos,SEEK_SET);
-                    (void) WriteBlobMSBLong(ofile,diff);
-                    (void) SeekBlob(ofile,currentpos,SEEK_SET);
+                    if (SeekBlob(ofile,savedpos,SEEK_SET) != savedpos)
+                      goto parse8BIMW_failure;
+                    if (WriteBlobMSBLong(ofile,diff) != 4)
+                      goto parse8BIMW_failure;
+                    if (SeekBlob(ofile,currentpos,SEEK_SET) != currentpos)
+                      goto parse8BIMW_failure;
                     savedolen = 0L;
                   }
                 if (outputlen & 1)
                   {
-                    (void) WriteBlobByte(ofile,0x00);
+                    if (WriteBlobByte(ofile,0x00) != 1)
+                      goto parse8BIMW_failure;
                     outputlen++;
                   }
-                (void) WriteBlobString(ofile,"8BIM");
-                (void) WriteBlobMSBShort(ofile,recnum);
+                if (WriteBlobString(ofile,"8BIM") < 4)
+                  goto parse8BIMW_failure;
+                if (WriteBlobMSBShort(ofile,recnum) < 2)
+                  goto parse8BIMW_failure;
                 outputlen += 6;
                 if (name)
                   nlen = (unsigned char) strlen(name);
-                (void) WriteBlobByte(ofile,nlen);
+                if (WriteBlobByte(ofile,nlen) != 1)
+                  goto parse8BIMW_failure;
                 outputlen++;
                 for (i=0; i<nlen; i++)
-                  (void) WriteBlobByte(ofile,name[i]);
+                  if (WriteBlobByte(ofile,name[i]) != 1)
+                    goto parse8BIMW_failure;
                 outputlen += nlen;
                 if (!(nlen&1))
                   {
-                    (void) WriteBlobByte(ofile,0x00);
+                    if (WriteBlobByte(ofile,0x00) != 1)
+                      goto parse8BIMW_failure;
                     outputlen++;
                   }
                 if (recnum != IPTC_ID)
                   {
-                    (void) WriteBlobMSBLong(ofile, (const magick_uint32_t) len);
+                    if (WriteBlobMSBLong(ofile, (magick_uint32_t) len) != 4)
+                      goto parse8BIMW_failure;
                     outputlen += 4;
 
                     next=0;
                     outputlen += len;
                     while (len--)
-                      (void) WriteBlobByte(ofile,token[next++]);
+                      if (WriteBlobByte(ofile,token[next++]) != 1)
+                        goto parse8BIMW_failure;
 
                     if (outputlen & 1)
                       {
-                        (void) WriteBlobByte(ofile,0x00);
+                        if (WriteBlobByte(ofile,0x00) != 1)
+                          goto parse8BIMW_failure;
                         outputlen++;
                       }
                   }
@@ -764,7 +835,8 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                     savedpos = TellBlob(ofile);
                     if (savedpos < 0)
                       goto parse8BIMW_failure;
-                    (void) WriteBlobMSBLong(ofile,0xFFFFFFFFUL);
+                    if (WriteBlobMSBLong(ofile,0xFFFFFFFFUL) != 4)
+                      goto parse8BIMW_failure;
                     outputlen += 4;
                     savedolen = outputlen;
                   }
@@ -773,25 +845,30 @@ static long parse8BIMW(Image *ifile, Image *ofile)
               {
                 if (len <= 0x7FFF)
                   {
-                    (void) WriteBlobByte(ofile,0x1c);
-                    (void) WriteBlobByte(ofile,dataset);
-                    (void) WriteBlobByte(ofile,recnum & 255);
-                    (void) WriteBlobMSBShort(ofile,(const magick_uint16_t) len);
+                    if (WriteBlobByte(ofile,0x1c) != 1)
+                      goto parse8BIMW_failure;
+                    if (WriteBlobByte(ofile,dataset) != 1)
+                      goto parse8BIMW_failure;
+                    if (WriteBlobByte(ofile,recnum & 255) != 1)
+                      goto parse8BIMW_failure;
+                    if (WriteBlobMSBShort(ofile,(magick_uint16_t) len) != 2)
+                      goto parse8BIMW_failure;
                     outputlen += 5;
                     next=0;
                     outputlen += (long) len;
                     while (len--)
-                      (void) WriteBlobByte(ofile,token[next++]);
+                      if (WriteBlobByte(ofile,token[next++]) != 1)
+                        goto parse8BIMW_failure;
                   }
               }
           }
       state++;
     }
-    MagickFreeMemory(token);
-    MagickFreeMemory(newstr);
-    MagickFreeMemory(name);
+    MagickFreeResourceLimitedMemory(char *,token);
+    MagickFreeResourceLimitedMemory(char *,newstr);
+    MagickFreeResourceLimitedMemory(char *,name);
   }
-  MagickFreeMemory(line);
+  MagickFreeResourceLimitedMemory(char *,line);
   if (savedolen > 0)
     {
       long diff = outputlen - savedolen;
@@ -799,18 +876,21 @@ static long parse8BIMW(Image *ifile, Image *ofile)
       currentpos = TellBlob(ofile);
       if (currentpos < 0)
         goto parse8BIMW_failure;
-      (void) SeekBlob(ofile,savedpos,SEEK_SET);
-      (void) WriteBlobMSBLong(ofile,diff);
-      (void) SeekBlob(ofile,currentpos,SEEK_SET);
+      if (SeekBlob(ofile,savedpos,SEEK_SET) != savedpos)
+        goto parse8BIMW_failure;
+      if (WriteBlobMSBLong(ofile,diff) != 4)
+        goto parse8BIMW_failure;
+      if (SeekBlob(ofile,currentpos,SEEK_SET) != currentpos)
+        goto parse8BIMW_failure;
       savedolen = 0L;
     }
   return outputlen;
 
  parse8BIMW_failure:
-  MagickFreeMemory(token);
-  MagickFreeMemory(newstr);
-  MagickFreeMemory(name);
-  MagickFreeMemory(line);
+  MagickFreeResourceLimitedMemory(char *,token);
+  MagickFreeResourceLimitedMemory(char *,newstr);
+  MagickFreeResourceLimitedMemory(char *,name);
+  MagickFreeResourceLimitedMemory(char *,line);
   return 0L;
 }
 
@@ -855,7 +935,8 @@ static int jpeg_transfer_1(Image *ifile, Image *ofile)
   c = ReadBlobByte(ifile);
   if (c == EOF)
     return EOF;
-  (void) WriteBlobByte(ofile,c);
+  if (WriteBlobByte(ofile,c) != 1)
+    return EOF;
   return c;
 }
 
@@ -929,7 +1010,8 @@ static int jpeg_nextmarker(Image *ifile, Image *ofile)
       return M_EOI; /* we hit EOF */
     else
       if (c != 0xff)
-        (void) WriteBlobByte(ofile,c);
+        if (WriteBlobByte(ofile,c) != 1)
+          return M_EOI; /* pretend we hit EOF */
   } while (c != 0xff);
 
   /* get marker byte, swallowing possible padding */
@@ -1097,7 +1179,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   size_t
     length;
 
-  unsigned int
+  MagickPassFail
     status;
 
   unsigned char
@@ -1112,7 +1194,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFail)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
   image->columns=1;
   image->rows=1;
@@ -1131,12 +1213,21 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         {
           DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
-            image)
+                               image);
         }
+      (void) memset(blob,0,length);
       AttachBlob(buff->blob,blob,length);
       if (LocaleCompare(image_info->magick,"8BIMTEXT") == 0)
         {
           length=parse8BIM(image, buff);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                "META CODER Parse8BIM returned: %lu",
+                                (unsigned long) length);
+          if (length == 0)
+            {
+              status = MagickFail;
+              goto t8bim_failure;
+            }
           if (length & 1)
             (void) WriteBlobByte(buff,0x0);
         }
@@ -1146,6 +1237,11 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                 "META CODER Parse8BIMW returned: %lu",
                                 (unsigned long) length);
+          if (length == 0)
+            {
+              status = MagickFail;
+              goto t8bim_failure;
+            }
           if (length & 1)
             (void) WriteBlobByte(buff,0x0);
         }
@@ -1159,15 +1255,19 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             (void) WriteBlobByte(buff,c);
           }
         }
-      blob=GetBlobStreamData(buff);
+      blob=GetBlobStreamData(buff);  /* blob may be realloced */
       length=GetBlobSize(buff);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                             "Store IPTC profile, size %lu bytes",
                             (unsigned long) length);
       (void) SetImageProfile(image,"IPTC",blob,length);
+    t8bim_failure:;
+      blob=GetBlobStreamData(buff); /* blob may be realloced */
       DetachBlob(buff->blob);
       MagickFreeMemory(blob);
       DestroyImage(buff);
+      if (status == MagickFail)
+        ThrowReaderException(CorruptImageError,CorruptImage,image);
     }
   if (LocaleNCompare(image_info->magick,"APP1",4) == 0)
     {
@@ -1182,7 +1282,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         {
           DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
-            image)
+                               image);
         }
       AttachBlob(buff->blob,blob,length);
       if (LocaleCompare(image_info->magick,"APP1JPEG") == 0)
@@ -1200,6 +1300,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           if ((pinfo == (ProfileInfo *) NULL) ||
               (pinfo->info == (unsigned char *) NULL) || (pinfo->length == 0))
             {
+              blob=GetBlobStreamData(buff); /* blob may be realloced */
               DetachBlob(buff->blob);
               MagickFreeMemory(blob);
               DestroyImage(buff);
@@ -1208,6 +1309,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           iptc=AllocateImage((ImageInfo *) NULL);
           if (iptc == (Image *) NULL)
             {
+              blob=GetBlobStreamData(buff); /* blob may be realloced */
               DetachBlob(buff->blob);
               MagickFreeMemory(blob);
               DestroyImage(buff);
@@ -1224,6 +1326,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           DestroyImage(iptc);
           if (result == 0)
             {
+              blob=GetBlobStreamData(buff); /* blob may be realloced */
               DetachBlob(buff->blob);
               MagickFreeMemory(blob);
               DestroyImage(buff);
@@ -1239,41 +1342,42 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             c=ReadBlobByte(image);
             if (c == EOF)
               break;
-            WriteBlobByte(buff,c);
+            if (WriteBlobByte(buff,c) != 1)
+              break;
           }
 #else
 #define MaxBufferSize  65541
           char
             *buffer;
 
-          ExtendedSignedIntegralType
-            count;
-
           size_t
+            count,
             i,
             length;
 
-          buffer=MagickAllocateMemory(char *,MaxBufferSize);
-          if (buffer != (char *) NULL)
+          buffer=MagickAllocateResourceLimitedMemory(char *,MaxBufferSize);
+          if (buffer == (char *) NULL)
             {
-              i=0;
-              while ((length=ReadBlob(image,MaxBufferSize,buffer)) != 0)
-              {
-                count=0;
-                for (i=0; i < length; i+=count)
+              DestroyImage(buff);
+              ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+            }
+          i=0;
+          while ((length=ReadBlob(image,MaxBufferSize,buffer)) != 0)
+            {
+              count=0;
+              for (i=0; i < length; i+=count)
                 {
                   count=WriteBlob(buff,length-i,buffer+i);
-                  if (count <= 0)
+                  if (count != (length-i))
                     break;
                 }
-                if (i < length)
-                  break;
-              }
-              MagickFreeMemory(buffer);
+              if (i < length)
+                break;
             }
+          MagickFreeResourceLimitedMemory(char *,buffer);
 #endif
         }
-      blob=GetBlobStreamData(buff);
+      blob=GetBlobStreamData(buff); /* blob may be realloced */
       length=GetBlobSize(buff);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                             "Store APP1 profile, size %lu bytes",
@@ -1294,7 +1398,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         {
           DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
-            image)
+                               image);
         }
       AttachBlob(buff->blob,blob,length);
       for ( ; ; )
@@ -1304,7 +1408,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           break;
         (void) WriteBlobByte(buff,c);
       }
-      blob=GetBlobStreamData(buff);
+      blob=GetBlobStreamData(buff); /* blob may be realloced */
       length=GetBlobSize(buff);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                             "Store ICM profile, size %lu bytes",
@@ -1346,7 +1450,8 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             c=ReadBlobByte(image);
             if (c == EOF)
               break;
-            (void) WriteBlobByte(buff,c);
+            if (WriteBlobByte(buff,c) != 1)
+              break;
           }
         }
 
@@ -1361,14 +1466,14 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       length=GetBlobSize(buff)-12;
       blob[10]=(unsigned char) ((length >> 8) & 0xff);
       blob[11]=(unsigned char) (length & 0xff);
-      blob=GetBlobStreamData(buff);
+      blob=GetBlobStreamData(buff); /* blob may be realloced */
       length=GetBlobSize(buff);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                             "Store IPTC profile, size %" MAGICK_SIZE_T_F
                             "u bytes",(MAGICK_SIZE_T) length);
       (void) SetImageProfile(image,"IPTC",blob,length);
       DetachBlob(buff->blob);
-      MagickFreeMemory(blob)
+      MagickFreeMemory(blob);
       DestroyImage(buff);
     }
   CloseBlob(image);
@@ -1615,12 +1720,12 @@ static size_t GetIPTCStream(const unsigned char *blob, size_t blob_length, size_
     tag_length,
     blob_remaining;
 
+  *offset=0;
   p=blob;
   blob_remaining=blob_length;
   if ((*p == 0x1c) && (*(p+1) == 0x02))
     {
       /* This looks like a plain IPTC record 2 block so drop through */
-      *offset=0;
       return blob_length;
     }
 
@@ -1748,7 +1853,7 @@ iptc_find:
     else
       {
         /* short format */
-        tag_length=((long) c) << 8;
+        tag_length=((size_t) c) << 8;
         c=(*p++);
         blob_length--;
         if (blob_length == 0)
@@ -1808,16 +1913,14 @@ static void formatString(Image *ofile, const char *s, int len)
 #endif
 }
 
-typedef struct _tag_spec
+static const struct
 {
-  short
+  unsigned char
     id;
 
   const char
-    *name;
-} tag_spec;
-
-static const tag_spec tags[] = {
+    name[32];
+} tags[] = {
   { 5, "Image Name" },
   { 7, "Edit Status" },
   { 10, "Priority" },
@@ -1896,7 +1999,7 @@ static int formatIPTC(Image *ifile, Image *ofile)
 
   int
     i,
-    tagcount = sizeof(tags) / sizeof(tag_spec);
+    tagcount = ArraySize(tags);
 
   int
     c;
@@ -1956,7 +2059,7 @@ static int formatIPTC(Image *ifile, Image *ofile)
       }
     if (taglen < 0) return -1;
     /* make a buffer to hold the tag data and snag it from the input stream */
-    str=MagickAllocateMemory(unsigned char *,(unsigned int) (taglen+1));
+    str=MagickAllocateResourceLimitedMemory(unsigned char *, (size_t) taglen+1);
     if (str == (unsigned char *) NULL)
       {
         (void) printf("MemoryAllocationFailed");
@@ -1967,7 +2070,7 @@ static int formatIPTC(Image *ifile, Image *ofile)
       c=ReadBlobByte(ifile);
       if (c == EOF)
         {
-          MagickFreeMemory(str);
+          MagickFreeResourceLimitedMemory(unsigned char *,str);
           return -1;
         }
       str[tagindx] = (unsigned char) c;
@@ -1979,9 +2082,13 @@ static int formatIPTC(Image *ifile, Image *ofile)
       FormatString(temp, "%d#%d#%s=",(unsigned int)dataset, (unsigned int) recnum, readable);
     else
       FormatString(temp, "%d#%d=",(unsigned int)dataset, (unsigned int) recnum);
-    (void) WriteBlobString(ofile,temp);
+    if (WriteBlobString(ofile,temp) == 0)
+      {
+        MagickFreeResourceLimitedMemory(unsigned char *,str);
+        return -1;
+      }
     formatString( ofile, (char *)str, taglen );
-    MagickFreeMemory(str);
+    MagickFreeResourceLimitedMemory(unsigned char *,str);
 
     tagsfound++;
 
@@ -2032,7 +2139,7 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, long len)
 
   int
     i,
-    tagcount = sizeof(tags) / sizeof(tag_spec);
+    tagcount = ArraySize(tags);
 
   int
     c;
@@ -2083,7 +2190,7 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, long len)
       }
     if (taglen < 0) return -1;
     /* make a buffer to hold the tag data and snag it from the input stream */
-    str=MagickAllocateMemory(unsigned char *,(unsigned int) (taglen+1));
+    str=MagickAllocateResourceLimitedMemory(unsigned char *,(size_t) taglen+1);
     if (str == (unsigned char *) NULL)
       {
         (void) printf("MemoryAllocationFailed");
@@ -2094,7 +2201,7 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, long len)
       c = *s++; len--;
       if (len < 0)
         {
-          MagickFreeMemory(str);
+          MagickFreeResourceLimitedMemory(unsigned char *,str);
           return -1;
         }
       str[tagindx] = (unsigned char) c;
@@ -2108,18 +2215,18 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, long len)
       FormatString(temp, "%d#%d=",(unsigned int)dataset, (unsigned int) recnum);
     (void) WriteBlobString(ofile,temp);
     formatString( ofile, (char *)str, taglen );
-    MagickFreeMemory(str);
+    MagickFreeResourceLimitedMemory(unsigned char *,str);
 
     tagsfound++;
   }
   return tagsfound;
 }
 
-#define Format8BIMLiberate()                    \
-  do                                            \
-    {                                           \
-      MagickFreeMemory(PString);                \
-      MagickFreeMemory(str);                    \
+#define Format8BIMLiberate()                                   \
+  do                                                           \
+    {                                                          \
+      MagickFreeResourceLimitedMemory(unsigned char *,PString);  \
+      MagickFreeResourceLimitedMemory(unsigned char *,str);      \
     } while(0);
 
 static int format8BIM(Image *ifile, Image *ofile)
@@ -2195,7 +2302,7 @@ static int format8BIM(Image *ifile, Image *ofile)
             goto format8BIMError;
           }
         plen = (unsigned char) c;
-        PString=MagickAllocateMemory(unsigned char *,(unsigned int) (plen+1));
+        PString=MagickAllocateResourceLimitedMemory(unsigned char *,(size_t) plen+1);
         if (PString == (unsigned char *) NULL)
           {
             ThrowException(&ofile->exception,ResourceLimitError,MemoryAllocationFailed,
@@ -2235,7 +2342,7 @@ static int format8BIM(Image *ifile, Image *ofile)
           goto format8BIMError;
         }
       /* make a buffer to hold the data and snag it from the input stream */
-      str=MagickAllocateMemory(unsigned char *,(size_t) Size+1);
+      str=MagickAllocateResourceLimitedMemory(unsigned char *,(size_t) Size+1);
       if (str == (unsigned char *) NULL)
         {
           ThrowException(&ofile->exception,ResourceLimitError,MemoryAllocationFailed,
@@ -2320,8 +2427,8 @@ static MagickPassFail WriteMETAImage(const ImageInfo *image_info,Image *image)
       if (status == MagickFail)
         ThrowWriterException(FileOpenError,UnableToOpenFile,image);
       (void) WriteBlob(image,profile_length,(void *) profile);
-      CloseBlob(image);
-      return MagickPass;
+      status &= CloseBlob(image);
+      return status;
     }
   if (LocaleCompare(image_info->magick,"IPTC") == 0)
     {
@@ -2336,14 +2443,12 @@ static MagickPassFail WriteMETAImage(const ImageInfo *image_info,Image *image)
         ThrowWriterException(CoderError,NoIPTCProfileAvailable,image);
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       length=GetIPTCStream(profile,profile_length,&iptc_offset);
-      info=profile+iptc_offset;
       if (length == 0)
-        {
-          ThrowWriterException(CoderError,NoIPTCInfoWasFound,image);
-        }
+        ThrowWriterException(CoderError,NoIPTCInfoWasFound,image);
+      info=profile+iptc_offset;
       (void) WriteBlob(image,length,info);
-      CloseBlob(image);
-      return MagickPass;
+      status &= CloseBlob(image);
+      return status;
     }
   if (LocaleCompare(image_info->magick,"8BIMTEXT") == 0)
     {
@@ -2366,7 +2471,7 @@ static MagickPassFail WriteMETAImage(const ImageInfo *image_info,Image *image)
         status = MagickFail;
       DetachBlob(buff->blob);
       DestroyImage(buff);
-      CloseBlob(image);
+      status &= CloseBlob(image);
       return status;
     }
   if (LocaleCompare(image_info->magick,"8BIMWTEXT") == 0)
@@ -2379,7 +2484,7 @@ static MagickPassFail WriteMETAImage(const ImageInfo *image_info,Image *image)
         *buff;
 
       size_t
-        iptc_offset,
+        iptc_offset = 0,
         length;
 
       const unsigned char
@@ -2388,9 +2493,9 @@ static MagickPassFail WriteMETAImage(const ImageInfo *image_info,Image *image)
       if((profile=GetImageProfile(image,"IPTC",&profile_length)) == 0)
         ThrowWriterException(CoderError,NoIPTCProfileAvailable,image);
       length=GetIPTCStream(profile,profile_length,&iptc_offset);
-      info=profile+iptc_offset;
       if (length == 0)
         ThrowWriterException(CoderError,NoIPTCInfoWasFound,image);
+      info=profile+iptc_offset;
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFail)
         ThrowWriterException(FileOpenError,UnableToOpenFile,image);
@@ -2400,11 +2505,11 @@ static MagickPassFail WriteMETAImage(const ImageInfo *image_info,Image *image)
           ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
         }
       AttachBlob(buff->blob,info,length);
-      (void) formatIPTC(buff,image);
+      (void) formatIPTC(buff,image); /* returns -1 on failure */
       DetachBlob(buff->blob);
       DestroyImage(buff);
-      CloseBlob(image);
-      return MagickPass;
+      status &= CloseBlob(image);
+      return status;
     }
   if (LocaleCompare(image_info->magick,"IPTCWTEXT") == 0)
     {
@@ -2423,8 +2528,8 @@ static MagickPassFail WriteMETAImage(const ImageInfo *image_info,Image *image)
       if (status == MagickFail)
         ThrowWriterException(FileOpenError,UnableToOpenFile,image);
       (void) WriteBlob(image,(int) profile_length, (char *) profile);
-      CloseBlob(image);
-      return MagickPass;
+      status &= CloseBlob(image);
+      return status;
     }
   if ((LocaleCompare(image_info->magick,"ICC") == 0) ||
       (LocaleCompare(image_info->magick,"ICM") == 0))
@@ -2439,8 +2544,8 @@ static MagickPassFail WriteMETAImage(const ImageInfo *image_info,Image *image)
       if (status == MagickFail)
         ThrowWriterException(FileOpenError,UnableToOpenFile,image);
       (void) WriteBlob(image,profile_length,(void *) profile);
-      CloseBlob(image);
-      return MagickPass;
+      status &= CloseBlob(image);
+      return status;
     }
   return MagickFail;
 }
