@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2020 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -17,7 +17,7 @@
 %                            M   M  P       CCCC                              %
 %                                                                             %
 %                                                                             %
-%              Read/Write Magick Persistant Cache Image Format.               %
+%              Read/Write Magick Persistent Cache Image Format.               %
 %                                                                             %
 %                                                                             %
 %                              Software Design                                %
@@ -47,6 +47,7 @@
 #include "magick/pixel_cache.h"
 #include "magick/profile.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -126,23 +127,43 @@ static MagickBool IsMPC(const unsigned char *magick,const size_t length)
 %
 */
 
-#define ThrowMPCReaderException(code_,reason_,image_) \
-do { \
-  MagickFreeMemory(comment); \
-  MagickFreeMemory(values); \
-  if (number_of_profiles > 0) \
-    { \
-      unsigned int _index; \
-      for (_index=0; _index < number_of_profiles; _index++) \
-        { \
-          MagickFreeMemory(profiles[_index].name); \
-          MagickFreeMemory(profiles[_index].info); \
-        } \
-      MagickFreeMemory(profiles); \
-      number_of_profiles=0; \
-    } \
-  ThrowReaderException(code_,reason_,image_); \
-} while (0);
+#define ThrowMPCReaderException(code_,reason_,image_)                   \
+  do {                                                                  \
+    MagickFreeResourceLimitedMemory(char *,comment);                    \
+    MagickFreeResourceLimitedMemory(char *,values);                     \
+    if (number_of_profiles > 0)                                         \
+      {                                                                 \
+        unsigned long _index;                                           \
+        for (_index=0; _index < number_of_profiles; _index++)           \
+          {                                                             \
+            MagickFreeMemory(profiles[_index].name);                    \
+            MagickFreeResourceLimitedMemory(unsigned char *,profiles[_index].info); \
+          }                                                             \
+        MagickFreeResourceLimitedMemory(ProfileInfo *,profiles);        \
+        number_of_profiles=0;                                           \
+      }                                                                 \
+    ThrowReaderException(code_,reason_,image_);                         \
+  } while (0);
+
+#define ReadMPCMaxKeyWordCount 256 /* Arbitrary limit on number of keywords in MPC frame */
+
+/*
+  Ignore attempts to set the same attribute multiple times.
+*/
+static MagickPassFail
+SetNewImageAttribute(Image *image,const char *key,const char *value)
+{
+  MagickPassFail
+    status;
+
+  if (GetImageAttribute(image,key) == (const ImageAttribute *) NULL)
+    status = SetImageAttribute(image,key,value);
+  else
+    status = MagickFail;
+
+  return status;
+};
+
 static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
@@ -159,7 +180,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   int
     c;
 
-  register long
+  register unsigned long
     i;
 
   register unsigned char
@@ -176,7 +197,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ProfileInfo
     *profiles=0;
 
-  unsigned int
+  unsigned long
     number_of_profiles=0;
 
   char
@@ -259,7 +280,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             Read comment-- any text between { }.
           */
           comment_length=MaxTextExtent;
-          comment=MagickAllocateMemory(char *,comment_length);
+          comment=MagickAllocateResourceLimitedMemory(char *,comment_length);
           if (comment == (char *) NULL)
             ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
               image);
@@ -271,11 +292,18 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             if ((size_t) (p-comment+1) >= comment_length)
               {
+                char
+                  *new_comment;
+
                 *p='\0';
-                comment_length<<=1;
-                MagickReallocMemory(char *,comment,comment_length);
-                if (comment == (char *) NULL)
-                  break;
+                comment_length += MaxTextExtent;
+                new_comment=MagickReallocateResourceLimitedMemory(char *,comment,comment_length);
+                if (new_comment == (char *) NULL)
+                  {
+                    MagickFreeResourceLimitedMemory(char *,comment);
+                    break;
+                  }
+                comment=new_comment;
                 p=comment+strlen(comment);
               }
             *p=c;
@@ -284,9 +312,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
               image);
           *p='\0';
-          (void) SetImageAttribute(image,"comment",comment);
+          (void) SetNewImageAttribute(image,"comment",comment);
           comment_count++;
-          MagickFreeMemory(comment);
+          MagickFreeResourceLimitedMemory(char *,comment);
           c=ReadBlobByte(image);
         }
       else
@@ -328,11 +356,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               Get values.
 
               Values not containing spaces are terminated by the first
-              white-space (or new-line) enountered.  Values containing
+              white-space (or new-line) encountered.  Values containing
               spaces and/or new-lines must be surrounded by braces.
             */
             values_length=MaxTextExtent;
-            values=MagickAllocateMemory(char *,values_length);
+            values=MagickAllocateResourceLimitedMemory(char *,values_length);
             if (values == (char *) NULL)
               ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,image);
             values[0]='\0';
@@ -347,11 +375,18 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if ((size_t) (p-values+1) >= values_length)
                   {
+                    char
+                      *new_values;
+
                     *p='\0';
-                    values_length<<=1;
-                    MagickReallocMemory(char *,values,values_length);
-                    if (values == (char *) NULL)
-                      break;
+                    values_length += MaxTextExtent;
+                    new_values=MagickReallocateResourceLimitedMemory(char *,values,values_length);
+                    if (new_values == (char *) NULL)
+                      {
+                        MagickFreeResourceLimitedMemory(char *,values);
+                        break;
+                      }
+                    values=new_values;
                     p=values+strlen(values);
                   }
                 if (values == (char *) NULL)
@@ -375,6 +410,16 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                       "First keyword/value must be 'id=MagickCache'");
                 ThrowMPCReaderException(CorruptImageError,ImproperImageHeader,image);
               }
+            /*
+              Arbitrarily limit the number of header keywords to avoid DOS attempts.
+            */
+            if (keyword_count > ReadMPCMaxKeyWordCount)
+              {
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                      "Excessive key word count %u"
+                                      " (Denial of service attempt?)",keyword_count);
+                ThrowMPCReaderException(CorruptImageError,ImproperImageHeader,image);
+              }
             /*
               Assign a value to the specified keyword.
             */
@@ -402,7 +447,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       exception);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -466,7 +511,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->columns= MagickAtoL(values);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -496,7 +541,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           image->dispose=PreviousDispose;
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -508,7 +553,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->error.mean_error_per_pixel=MagickAtoF(values);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -533,7 +578,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       &image->chromaticity.green_primary.y);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -550,7 +595,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->iterations=MagickAtoL(values);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -590,7 +635,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     (void) CloneString(&image->montage,values);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -608,7 +653,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->orientation=StringToOrientationType(values);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -628,20 +673,23 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleNCompare(keyword,"profile-",8) == 0)
                   {
-                    i=(long) number_of_profiles;
-                    MagickReallocMemory(ProfileInfo *,profiles,MagickArraySize((size_t) i+1,sizeof(ProfileInfo)));
-                    if (profiles == (ProfileInfo *) NULL)
-                      {
-                        MagickFreeMemory(values);
-                        ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-                      }
+                    ProfileInfo
+                      *new_profiles;
+
+                    i=number_of_profiles;
+
+                    new_profiles=MagickReallocateResourceLimitedArray(ProfileInfo *,profiles,
+                        (size_t)number_of_profiles+1,sizeof(ProfileInfo));
+                    if (new_profiles == (ProfileInfo *) NULL)
+                      ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                    profiles=new_profiles;
                     profiles[i].name=AllocateString(keyword+8);
                     profiles[i].length=MagickAtoL(values);
                     profiles[i].info=(unsigned char *) NULL;
                     number_of_profiles++;
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -653,7 +701,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     quantum_depth=MagickAtoL(values);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -694,7 +742,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->rows=MagickAtoL(values);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -706,7 +754,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->scene=MagickAtoL(values);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -723,7 +771,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         image->units=PixelsPerCentimeterResolution;
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
@@ -737,18 +785,18 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       &image->chromaticity.white_point.y);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
               default:
               {
-                (void) SetImageAttribute(image,keyword,
+                (void) SetNewImageAttribute(image,keyword,
                   *values == '{' ? values+1 : values);
                 break;
               }
             }
-            MagickFreeMemory(values);
+            MagickFreeResourceLimitedMemory(char *,values);
           }
         else
           {
@@ -821,7 +869,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     if (number_of_profiles > 0)
       {
-        for (i=0; i < (long) number_of_profiles; i++)
+        for (i=0; i < number_of_profiles; i++)
         {
           if (profiles[i].length > 0)
             {
@@ -831,15 +879,15 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     (magick_off_t) profiles[i].length) ||
                    (profiles[i].length < 15*1024*1024)))
                 {
-                  profiles[i].info=MagickAllocateMemory(unsigned char *,profiles[i].length);
+                  profiles[i].info=MagickAllocateResourceLimitedMemory(unsigned char *,profiles[i].length);
                   if (profiles[i].info == (unsigned char *) NULL)
-                    ThrowMPCReaderException(CorruptImageError,UnableToReadGenericProfile,
+                    ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
                                              image);
                   if (ReadBlob(image,profiles[i].length,profiles[i].info)
                       != profiles[i].length)
                     ThrowMPCReaderException(CorruptImageError,
-                                             UnexpectedEndOfFile,
-                                             image);
+                                            UnexpectedEndOfFile,
+                                            image);
                   (void) SetImageProfile(image,profiles[i].name,profiles[i].info,profiles[i].length);
                 }
               else
@@ -852,9 +900,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 }
             }
           MagickFreeMemory(profiles[i].name);
-          MagickFreeMemory(profiles[i].info);
+          MagickFreeResourceLimitedMemory(unsigned char *,profiles[i].info);
         }
-        MagickFreeMemory(profiles);
+        MagickFreeResourceLimitedMemory(ProfileInfo *,profiles);
         number_of_profiles=0;
       }
 
@@ -867,7 +915,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
         if (image->colors == 0)
-          for (i=0; i < 256; i++)
+          for (i=0; i < 256U; i++)
           {
             image->colormap[i].red=ScaleCharToQuantum(i);
             image->colormap[i].green=ScaleCharToQuantum(i);
@@ -886,21 +934,21 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               Read image colormap from file.
             */
             packet_size=image->depth > 8 ? 6 : 3;
-            colormap=MagickAllocateArray(unsigned char *,packet_size,image->colors);
+            colormap=MagickAllocateResourceLimitedArray(unsigned char *,packet_size,image->colors);
             if (colormap == (unsigned char *) NULL)
               ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
                 image);
             (void) ReadBlob(image, (size_t) packet_size*image->colors,colormap);
             p=colormap;
             if (image->depth <= 8)
-              for (i=0; i < (long) image->colors; i++)
+              for (i=0; i < image->colors; i++)
               {
                 image->colormap[i].red=ScaleCharToQuantum(*p++);
                 image->colormap[i].green=ScaleCharToQuantum(*p++);
                 image->colormap[i].blue=ScaleCharToQuantum(*p++);
               }
             else
-              for (i=0; i < (long) image->colors; i++)
+              for (i=0; i < image->colors; i++)
               {
                 image->colormap[i].red=(*p++ << 8);
                 image->colormap[i].red|=(*p++);
@@ -909,7 +957,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 image->colormap[i].blue=(*p++ << 8);
                 image->colormap[i].blue|=(*p++);
               }
-            MagickFreeMemory(colormap);
+            MagickFreeResourceLimitedMemory(unsigned char *,colormap);
           }
       }
     if (EOFBlob(image))
@@ -1003,7 +1051,9 @@ ModuleExport void RegisterMPCImage(void)
   entry->encoder=(EncoderHandler) WriteMPCImage;
   entry->magick=(MagickHandler) IsMPC;
   entry->description="Magick Persistent Cache image format";
+  entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->blob_support=MagickFalse;
   entry->module="MPC";
   entry->coder_class=UnstableCoderClass;
   (void) RegisterMagickInfo(entry);
@@ -1075,7 +1125,7 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
   ExtendedSignedIntegralType
     offset;
 
-  register long
+  register unsigned long
     i;
 
   unsigned int
@@ -1203,11 +1253,11 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
         /*
           Set image resolution.
         */
-        (void) strcpy(units,"undefined");
+        (void) strlcpy(units,"undefined",sizeof(units));
         if (image->units == PixelsPerInchResolution)
-          (void) strcpy(units,"pixels-per-inch");
+          (void) strlcpy(units,"pixels-per-inch",sizeof(units));
         if (image->units == PixelsPerCentimeterResolution)
-          (void) strcpy(units,"pixels-per-centimeter");
+          (void) strlcpy(units,"pixels-per-centimeter",sizeof(units));
         FormatString(buffer,"Resolution=%gx%g  units=%.1024s\n",
           image->x_resolution,image->y_resolution,units);
         (void) WriteBlobString(image,buffer);
@@ -1249,12 +1299,12 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
     if (image->dispose != UndefinedDispose)
       {
         if (image->dispose == BackgroundDispose)
-          (void) strcpy(buffer,"dispose=background\n");
+          (void) strlcpy(buffer,"dispose=background\n",sizeof(buffer));
         else
           if (image->dispose == NoneDispose)
-            (void) strcpy(buffer,"dispose=none\n");
+            (void) strlcpy(buffer,"dispose=none\n",sizeof(buffer));
           else
-            (void) strcpy(buffer,"dispose=previous\n");
+            (void) strlcpy(buffer,"dispose=previous\n",sizeof(buffer));
         (void) WriteBlobString(image,buffer);
       }
     if (image->error.mean_error_per_pixel != 0.0)
@@ -1391,7 +1441,7 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
           Allocate colormap.
         */
         packet_size=image->depth > 8 ? 6 : 3;
-        colormap=MagickAllocateArray(unsigned char *,packet_size,image->colors);
+        colormap=MagickAllocateResourceLimitedArray(unsigned char *,packet_size,image->colors);
         if (colormap == (unsigned char *) NULL)
           return(MagickFail);
         /*
@@ -1399,7 +1449,7 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
         */
         q=colormap;
         if (image->depth <= 8)
-          for (i=0; i < (long) image->colors; i++)
+          for (i=0; i < image->colors; i++)
           {
             *q++=image->colormap[i].red;
             *q++=image->colormap[i].green;
@@ -1407,7 +1457,7 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
           }
 #if QuantumDepth > 8
         else
-          for (i=0; i < (long) image->colors; i++)
+          for (i=0; i < image->colors; i++)
           {
             *q++=image->colormap[i].red >> 8;
             *q++=image->colormap[i].red & 0xff;
@@ -1419,7 +1469,7 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
 #endif /* QuantumDepth > 8 */
 
         (void) WriteBlob(image, (size_t) packet_size*image->colors,colormap);
-        MagickFreeMemory(colormap);
+        MagickFreeResourceLimitedMemory(unsigned char *,colormap);
       }
     /*
       Initialize persistent pixel cache.
@@ -1439,6 +1489,6 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
       image=image->previous;
-  CloseBlob(image);
+  status &= CloseBlob(image);
   return(status);
 }
