@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2020 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -42,6 +42,7 @@
 #include "magick/tempfile.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -300,7 +301,9 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
           }
         }
       (void) fclose(parameter_file);
+#if !defined(__COVERITY__) /* 384796 Unused value */
       parameter_file=(FILE *) NULL;
+#endif /* if !defined(__COVERITY__) */
     }
   if (image_info->quality == DefaultCompressionQuality)
     (void) fprintf(file,"-\n");  /* default non intra quant matrix */
