@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2020 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -49,6 +49,7 @@
 #include "magick/shear.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -214,7 +215,7 @@ static MagickPassFail DecodeImage(Image *image,unsigned char *luma,
   } PCDTable;
 
   long
-    count,
+    /* count, */
     quantum;
 
   PCDTable
@@ -326,7 +327,7 @@ static MagickPassFail DecodeImage(Image *image,unsigned char *luma,
   /*
     Recover the Huffman encoded luminance and chrominance deltas.
   */
-  count=0;
+  /* count=0; */
   length=0;
   plane=0;
   row=0;
@@ -350,20 +351,20 @@ static MagickPassFail DecodeImage(Image *image,unsigned char *luma,
           case 0:
           {
             q=luma+row*(size_t)image->columns;
-            count=(long) image->columns;
+            /* count=(long) image->columns; */
             break;
           }
           case 2:
           {
             q=chroma1+(row >> 1)*(size_t)image->columns;
-            count=(long) (image->columns >> 1);
+            /* count=(long) (image->columns >> 1); */
             plane--;
             break;
           }
           case 3:
           {
             q=chroma2+(row >> 1)*(size_t)image->columns;
-            count=(long) (image->columns >> 1);
+            /* count=(long) (image->columns >> 1); */
             plane--;
             break;
           }
@@ -413,15 +414,15 @@ static MagickPassFail DecodeImage(Image *image,unsigned char *luma,
                         (unsigned char) quantum);
     q++;
     PCDGetBits(r->length);
-    count--;
+    /* count--; */
   }
  decode_image_error:
   /*
     Free memory.
   */
   for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
-    MagickFreeResourceLimitedMemory(pcd_table[i]);
-  MagickFreeResourceLimitedMemory(buffer);
+    MagickFreeResourceLimitedMemory(PCDTable *,pcd_table[i]);
+  MagickFreeResourceLimitedMemory(unsigned char *,buffer);
   return(status);
 }
 
@@ -528,12 +529,12 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *images,
   return(montage_image);
 }
 
-#define ThrowPCDReaderException(code_,reason_,image_) \
-  {                                                   \
-    MagickFreeResourceLimitedMemory(chroma1);                        \
-    MagickFreeResourceLimitedMemory(chroma2);                        \
-    MagickFreeResourceLimitedMemory(luma);                           \
-    ThrowReaderException(code_,reason_,image_);       \
+#define ThrowPCDReaderException(code_,reason_,image_)                \
+  {                                                                  \
+    MagickFreeResourceLimitedMemory(unsigned char *,chroma1);        \
+    MagickFreeResourceLimitedMemory(unsigned char *,chroma2);        \
+    MagickFreeResourceLimitedMemory(unsigned char *,luma);           \
+    ThrowReaderException(code_,reason_,image_);                      \
   }
 
 static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
@@ -768,9 +769,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                     image->columns,image->rows))
           break;
       }
-      MagickFreeResourceLimitedMemory(chroma2);
-      MagickFreeResourceLimitedMemory(chroma1);
-      MagickFreeResourceLimitedMemory(luma);
+      MagickFreeResourceLimitedMemory(unsigned char *,chroma2);
+      MagickFreeResourceLimitedMemory(unsigned char *,chroma1);
+      MagickFreeResourceLimitedMemory(unsigned char *,luma);
       while (image->previous != (Image *) NULL)
         image=image->previous;
       CloseBlob(image);
@@ -869,9 +870,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                   image->columns,image->rows))
         break;
   }
-  MagickFreeResourceLimitedMemory(chroma2);
-  MagickFreeResourceLimitedMemory(chroma1);
-  MagickFreeResourceLimitedMemory(luma);
+  MagickFreeResourceLimitedMemory(unsigned char *,chroma2);
+  MagickFreeResourceLimitedMemory(unsigned char *,chroma1);
+  MagickFreeResourceLimitedMemory(unsigned char *,luma);
   if (LocaleCompare(image_info->magick,"PCDS") == 0)
     image->colorspace=sRGBColorspace;
   else
