@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2020 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -44,6 +44,7 @@
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Typedef declarations.
@@ -193,13 +194,14 @@ static unsigned int IsPCX(const unsigned char *magick,const size_t length)
 %
 %
 */
-#define ThrowPCXReaderException(code_,reason_,image_) \
-{ \
-  MagickFreeResourceLimitedMemory(page_table) \
-  MagickFreeResourceLimitedMemory(pcx_pixels); \
-  MagickFreeResourceLimitedMemory(scanline); \
-  ThrowReaderException(code_,reason_,image_); \
-}
+#define ThrowPCXReaderException(code_,reason_,image_)                   \
+  {                                                                     \
+    MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,page_table) \
+    MagickFreeResourceLimitedMemory(unsigned char *,pcx_pixels);        \
+    MagickFreeResourceLimitedMemory(unsigned char *,scanline);          \
+    ThrowReaderException(code_,reason_,image_);                         \
+  }
+
 static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
@@ -773,8 +775,8 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                       image->columns,image->rows))
             break;
     }
-    MagickFreeResourceLimitedMemory(scanline);
-    MagickFreeResourceLimitedMemory(pcx_pixels);
+    MagickFreeResourceLimitedMemory(unsigned char *,scanline);
+    MagickFreeResourceLimitedMemory(unsigned char *,pcx_pixels);
     if (EOFBlob(image))
       {
         ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,
@@ -814,7 +816,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
   }
   if (page_table != (ExtendedSignedIntegralType *) NULL)
-    MagickFreeResourceLimitedMemory(page_table);
+    MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,page_table);
   while (image->previous != (Image *) NULL)
     image=image->previous;
   CloseBlob(image);
@@ -924,7 +926,7 @@ ModuleExport void UnregisterPCXImage(void)
 %
 %
 */
-static MagickPassFail WriteRLEPixels(Image *image,
+static MagickPassFail WritePCXPixels(Image *image,
                                      PCXInfo *pcx_info,
                                      const unsigned char *pcx_row_pixels)
 {
@@ -945,16 +947,32 @@ static MagickPassFail WriteRLEPixels(Image *image,
   /* For each color plane ... */
   for (i=0; i < (long) pcx_info->planes; i++)
     {
-      previous=(*q++);
-      count=1;
-      /* For each column ... */
-      for (x=0; x < (long) (pcx_info->bytes_per_line-1); x++)
+      if (pcx_info->encoding == 0)
         {
-          packet=(*q++);
-          if ((packet == previous) && (count < 63))
+          for (x=0; x < (long) pcx_info->bytes_per_line; x++)
+            (void) WriteBlobByte(image,(unsigned char) (*q++));
+        }
+      else
+        {
+          previous=(*q++);
+          count=1;
+          /* For each column ... */
+          for (x=0; x < (long) (pcx_info->bytes_per_line-1); x++)
             {
-              count++;
-              continue;
+              packet=(*q++);
+              if ((packet == previous) && (count < 63))
+                {
+                  count++;
+                  continue;
+                }
+              if ((count > 1) || ((previous & 0xc0) == 0xc0))
+                {
+                  count|=0xc0;
+                  (void) WriteBlobByte(image,count);
+                }
+              (void) WriteBlobByte(image,previous);
+              previous=packet;
+              count=1;
             }
           if ((count > 1) || ((previous & 0xc0) == 0xc0))
             {
@@ -962,31 +980,23 @@ static MagickPassFail WriteRLEPixels(Image *image,
               (void) WriteBlobByte(image,count);
             }
           (void) WriteBlobByte(image,previous);
-          previous=packet;
-          count=1;
         }
-      if ((count > 1) || ((previous & 0xc0) == 0xc0))
-        {
-          count|=0xc0;
-          (void) WriteBlobByte(image,count);
-        }
-      (void) WriteBlobByte(image,previous);
     }
   return (MagickPass);
 }
 
-#define LiberatePCXAllocations()                \
-  {                                             \
-    MagickFreeResourceLimitedMemory(pcx_colormap);             \
-    MagickFreeResourceLimitedMemory(pcx_pixels);               \
-    MagickFreeResourceLimitedMemory(page_table);               \
+#define LiberatePCXAllocations()                                        \
+  {                                                                     \
+    MagickFreeResourceLimitedMemory(unsigned char *,pcx_colormap);      \
+    MagickFreeResourceLimitedMemory(unsigned char *,pcx_pixels);        \
+    MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,page_table); \
   }
 
 #define ThrowPCXWriterException(code_,reason_,image_) \
-{ \
-  LiberatePCXAllocations();                   \
-  ThrowWriterException(code_,reason_,image_); \
-}
+  {                                                   \
+    LiberatePCXAllocations();                         \
+    ThrowWriterException(code_,reason_,image_);       \
+  }
 static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
 {
   long
@@ -1029,6 +1039,9 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
   unsigned long
     scene;
 
+  const unsigned long
+    max_scenes = 1024UL;
+
   ImageCharacteristics
     characteristics;
 
@@ -1057,11 +1070,12 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
       */
       write_dcx=MagickTrue;
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
-      page_table=MagickAllocateResourceLimitedMemory(ExtendedSignedIntegralType *,
-        1024*sizeof(ExtendedSignedIntegralType));
+      page_table=MagickAllocateResourceLimitedClearedArray(ExtendedSignedIntegralType *,
+                                                           (size_t) max_scenes+1,
+                                                           sizeof(ExtendedSignedIntegralType));
       if (page_table == (ExtendedSignedIntegralType *) NULL)
         ThrowPCXWriterException(ResourceLimitError,MemoryAllocationFailed,image);
-      for (scene=0; scene < 1024; scene++)
+      for (scene=0; scene < max_scenes; scene++)
         (void) WriteBlobLSBLong(image,0x00000000L);
     }
   adjoin=(image_info->adjoin) && (image->next != (const Image *) NULL) && (write_dcx);
@@ -1093,7 +1107,12 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     */
     pcx_info.identifier=0x0a;
     pcx_info.version=5;
-    pcx_info.encoding=1;
+    /* Please note that uncompressed PCX in quite rare and some applications cannot open it.
+       So the compressed PCX needs to be default. */
+    pcx_info.encoding = (image->compression==RLECompression || image->compression==UndefinedCompression) ? 1 : 0;
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Using %s compression",
+                          pcx_info.encoding == 1 ? "RLE" : "No");
     pcx_info.bits_per_pixel=8;
     if (characteristics.palette && characteristics.monochrome)
       pcx_info.bits_per_pixel=1;
@@ -1156,11 +1175,9 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     /*
       Dump colormap to file.
     */
-    pcx_colormap=MagickAllocateResourceLimitedMemory(unsigned char *,3*256);
+    pcx_colormap=MagickAllocateResourceLimitedClearedArray(unsigned char *,3,256);
     if (pcx_colormap == (unsigned char *) NULL)
       ThrowPCXWriterException(ResourceLimitError,MemoryAllocationFailed,image);
-    for (i=0; i < (3*256); i++)
-      pcx_colormap[i]=0;
     q=pcx_colormap;
     if (image->storage_class == PseudoClass)
       for (i=0; i < (long) image->colors; i++)
@@ -1177,7 +1194,9 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     for (i=0; i < 58; i++)
       (void) WriteBlobByte(image,'\0');
     /* Allocate memory for one pixel row. */
-    pcx_pixels=MagickAllocateResourceLimitedArray(unsigned char *,bytes_per_line,pcx_info.planes);
+    pcx_pixels=MagickAllocateResourceLimitedClearedArray(unsigned char *,
+                                                         bytes_per_line,
+                                                         pcx_info.planes);
     if (pcx_pixels == (unsigned char *) NULL)
       ThrowPCXWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     q=pcx_pixels;
@@ -1236,7 +1255,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
                 }
               }
           }
-          if (WriteRLEPixels(image,&pcx_info,pcx_pixels) == MagickFail)
+          if (WritePCXPixels(image,&pcx_info,pcx_pixels) == MagickFail)
             break;
           if (QuantumTick(y,image->rows))
             if (!MagickMonitorFormatted(y,image->rows,&image->exception,
@@ -1261,7 +1280,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
           /* For each column ... */
           for (x=0; x < (long) image->columns; x++)
             *q++=indexes[x];
-          if (WriteRLEPixels(image,&pcx_info,pcx_pixels) == MagickFail)
+          if (WritePCXPixels(image,&pcx_info,pcx_pixels) == MagickFail)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
@@ -1311,7 +1330,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             }
             if (bit != 0)
               *q++=byte << (8-bit);
-            if (WriteRLEPixels(image,&pcx_info,pcx_pixels) == MagickFail)
+            if (WritePCXPixels(image,&pcx_info,pcx_pixels) == MagickFail)
             break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
@@ -1324,17 +1343,17 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
 
     (void) WriteBlobByte(image,pcx_info.colormap_signature);
     (void) WriteBlob(image,3*256,(char *) pcx_colormap);
-    MagickFreeResourceLimitedMemory(pcx_pixels);
-    MagickFreeResourceLimitedMemory(pcx_colormap);
+    MagickFreeResourceLimitedMemory(unsigned char *,pcx_pixels);
+    MagickFreeResourceLimitedMemory(unsigned char *,pcx_colormap);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitorFormatted(scene++,image_list_length,
+    status=MagickMonitorFormatted(scene++,Min(max_scenes,image_list_length),
                                   &image->exception,SaveImagesText,
                                   image->filename);
     if (status == False)
       break;
-    if (scene >= 1023)
+    if (scene >= max_scenes-1)
       break;
   } while (adjoin);
   if (adjoin)
@@ -1345,15 +1364,19 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
       /*
         Write the DCX page table.
       */
+      if (logging && write_dcx && image_list_length > max_scenes)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "WARNING: DCX truncated to %lu scenes!",
+                              max_scenes-1);
       page_table[scene+1]=0;
       (void) SeekBlob(image,0L,SEEK_SET);
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
       for (i=0; i <= (long) scene; i++)
         (void) WriteBlobLSBLong(image,(unsigned long) page_table[i]);
-      MagickFreeResourceLimitedMemory(page_table);
+      MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,page_table);
     }
   if (status == False)
     ThrowPCXWriterException(FileOpenError,UnableToWriteFile,image);
-  CloseBlob(image);
-  return(True);
+  status &= CloseBlob(image);
+  return(status);
 }
