@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2019 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -48,6 +48,7 @@
 #include "magick/monitor.h"
 #include "magick/tempfile.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -126,6 +127,7 @@ static unsigned int IsPS(const unsigned char *magick,const size_t length)
 %
 %
 */
+#if defined(HasGS)
 static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define BoundingBox  "%%BoundingBox:"
@@ -240,6 +242,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (i=0; i < (long) (count-12); i++)
         (void) ReadBlobByte(image);
     }
+  bounds.x1=0.0;
+  bounds.y1=0.0;
+  bounds.x2=0.0;
+  bounds.y2=0.0;
   p=command;
   for (i=0; (LocaleCompare(image_info->magick,"EPT") != 0) ||
     (i < (long) filesize); i++)
@@ -261,10 +267,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       count=sscanf(command,"%%%%BoundingBox: %lf %lf %lf %lf",&bounds.x1,
         &bounds.y1,&bounds.x2,&bounds.y2);
     if (LocaleNCompare(DocumentMedia,command,strlen(DocumentMedia)) == 0)
-      count=sscanf(command,"%%%%DocumentMedia: %*s %lf %lf",&bounds.x2,
+      count=(size_t) sscanf(command,"%%%%DocumentMedia: %*s %lf %lf",&bounds.x2,
         &bounds.y2)+2;
     if (LocaleNCompare(PageBoundingBox,command,strlen(PageBoundingBox)) == 0)
-      count=sscanf(command,"%%%%PageBoundingBox: %lf %lf %lf %lf",
+      count=(size_t) sscanf(command,"%%%%PageBoundingBox: %lf %lf %lf %lf",
         &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
     if (count != 4)
       continue;
@@ -372,7 +378,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         *clone_info;
 
       clone_info=CloneImageInfo(image_info);
-      clone_info->blob=(void *) NULL;
+      clone_info->blob=(_BlobInfoPtr_) NULL;
       clone_info->length=0;
       clone_info->magick[0]='\0';
       clone_info->subimage=0;
@@ -382,16 +388,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       DestroyImageInfo(clone_info);
     }
   (void) LiberateTemporaryFile((char *) image_info->filename);
-#if defined(HasDPS)
-  if (image == (Image *) NULL)
-    {
-      /*
-        Ghostscript has failed-- try the Display Postscript Extension.
-      */
-      (void) FormatString((char *) image_info->filename,"dps:%.1024s",filename);
-      image=ReadImage(image_info,exception);
-    }
-#endif /* defined(HasDPS) */
   if (image == (Image *) NULL)
     {
       if (UndefinedException == exception->severity)
@@ -422,6 +418,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   return(image);
 }
+#endif /* if defined(HasGS) */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -452,7 +449,9 @@ ModuleExport void RegisterPSImage(void)
     *entry;
 
   entry=SetMagickInfo("EPI");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadPSImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WritePSImage;
   entry->magick=(MagickHandler) IsPS;
   entry->adjoin=False;
@@ -462,7 +461,9 @@ ModuleExport void RegisterPSImage(void)
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("EPS");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadPSImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WritePSImage;
   entry->magick=(MagickHandler) IsPS;
   entry->adjoin=False;
@@ -472,7 +473,9 @@ ModuleExport void RegisterPSImage(void)
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("EPSF");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadPSImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WritePSImage;
   entry->magick=(MagickHandler) IsPS;
   entry->adjoin=False;
@@ -482,7 +485,9 @@ ModuleExport void RegisterPSImage(void)
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("EPSI");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadPSImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WritePSImage;
   entry->magick=(MagickHandler) IsPS;
   entry->adjoin=False;
@@ -492,7 +497,9 @@ ModuleExport void RegisterPSImage(void)
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("PS");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadPSImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WritePSImage;
   entry->magick=(MagickHandler) IsPS;
   entry->description="Adobe PostScript";
@@ -787,6 +794,9 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
         /*
           Output Postscript header.
         */
+#if defined(HAVE_CTIME_R)
+        char time_buf[26];
+#endif /* defined(HAVE_CTIME_R) */
         if (LocaleCompare(image_info->magick,"PS") == 0)
           (void) strlcpy(buffer,"%!PS-Adobe-3.0\n",sizeof(buffer));
         else
@@ -796,15 +806,18 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
         FormatString(buffer,"%%%%Title: (%.1024s)\n",image->filename);
         (void) WriteBlobString(image,buffer);
         timer=time((time_t *) NULL);
-        (void) localtime(&timer);
-        (void) strlcpy(date,ctime(&timer),MaxTextExtent);
+#if defined(HAVE_CTIME_R)
+        (void) strlcpy(date,ctime_r(&timer,time_buf),MaxTextExtent);
+#else
+        (void) strlcpy(date,ctime(&timer),MaxTextExtent); /* Thread-unsafe version */
+#endif /* defined(HAVE_CTIME_R) */
         date[strlen(date)-1]='\0';
         FormatString(buffer,"%%%%CreationDate: (%.1024s)\n",date);
         (void) WriteBlobString(image,buffer);
         bounds.x1=geometry.x;
         bounds.y1=geometry.y;
         bounds.x2=geometry.x+x_scale;
-        bounds.y2=geometry.y+(geometry.height+text_size);
+        bounds.y2=geometry.y+((size_t) geometry.height+text_size);
         if (image_info->adjoin && (image->next != (Image *) NULL))
           (void) strlcpy(buffer,"%%%%BoundingBox: (atend)\n",sizeof(buffer));
         else
@@ -1200,10 +1213,10 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
       bounds.x1=geometry.x;
     if (geometry.y < bounds.y1)
       bounds.y1=geometry.y;
-    if ((geometry.x+(long) geometry.width-1) > bounds.x2)
-      bounds.x2=geometry.x+geometry.width-1;
-    if ((geometry.y+(long) (geometry.height+text_size)-1) > bounds.y2)
-      bounds.y2=geometry.y+(geometry.height+text_size)-1;
+    if ((geometry.x+((size_t) geometry.width-1)) > bounds.x2)
+      bounds.x2=geometry.x+(size_t) geometry.width-1;
+    if ((geometry.y+((size_t) geometry.height+text_size)-1) > bounds.y2)
+      bounds.y2=geometry.y+((size_t) geometry.height+text_size)-1;
     attribute=GetImageAttribute(image,"label");
     if (attribute != (const ImageAttribute *) NULL)
       (void) WriteBlobString(image,"%%%%PageResources: font Times-Roman\n");
@@ -1252,6 +1265,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             /*
               Dump image as grayscale.
             */
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Writing image as grayscale...");
             i++;
             bp=buffer;
             for (y=0; y < (long) image->rows; y++)
@@ -1300,11 +1315,9 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             /*
               Dump image as bitmap.
             */
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Writing image as monochrome bitmap...");
             (void) SetImageType(image,BilevelType);
-            polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
-            if (image->colors == 2)
-              polarity=PixelIntensityToQuantum(&image->colormap[1]) <
-                PixelIntensityToQuantum(&image->colormap[0]);
             count=0;
             bp=buffer;
             for (y=0; y < (long) image->rows; y++)
@@ -1319,8 +1332,16 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               for (x=0; x < (long) image->columns; x++)
               {
                 byte<<=1;
-                if (indexes[x] != polarity)
-                  byte|=0x01;
+                if (image->storage_class == PseudoClass)
+                  {
+                    if (GetGraySample(&image->colormap[indexes[x]]) >= MaxRGB/2)
+                      byte|=0x01;
+                  }
+                else
+                  {
+                    if (GetGraySample(p) >= MaxRGB/2)
+                      byte|=0x01;
+                  }
                 bit++;
                 if (bit == 8)
                   {
@@ -1383,6 +1404,9 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               /*
                 Dump runlength-encoded DirectColor packets.
               */
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                    "Writing image as DirectColor "
+                                    "(RLE compressed)...");
               bp=buffer;
               for (y=0; y < (long) image->rows; y++)
               {
@@ -1438,6 +1462,9 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               /*
                 Dump uncompressed DirectColor packets.
               */
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                    "Writing image as DirectColor "
+                                    "(uncompressed)...");
               i=0;
               bp=buffer;
               for (y=0; y < (long) image->rows; y++)
@@ -1512,6 +1539,10 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               /*
                 Dump runlength-encoded PseudoColor packets.
               */
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                    "Writing image as PseudoColor "
+                                    "(%u colors, RLE compressed)...",
+                                    image->colors);
               i=0;
               bp=buffer;
               for (y=0; y < (long) image->rows; y++)
@@ -1571,6 +1602,10 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               /*
                 Dump uncompressed PseudoColor packets.
               */
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                    "Writing image as PseudoColor "
+                                    "(%u colors, unompressed)...",
+                                    image->colors);
               i=0;
               bp=buffer;
               for (y=0; y < (long) image->rows; y++)
@@ -1631,6 +1666,6 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
-  CloseBlob(image);
-  return(True);
+  status &= CloseBlob(image);
+  return(status);
 }
