@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2015 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -51,6 +51,7 @@
 #include "magick/tempfile.h"
 #include "magick/utility.h"
 #include "magick/version.h"
+#include "magick/static.h"
 #if defined(HasTIFF)
 #define CCITTParam  "-1"
 #else
@@ -84,6 +85,14 @@ static unsigned int
 #include "tiffconf.h"
 #endif
 #include "tiffio.h"
+
+#if (TIFFLIB_VERSION >= 20201219)
+#  undef uint16
+#  define uint16 uint16_t
+#  undef uint32
+#  define uint32 uint32_t
+#endif /* TIFFLIB_VERSION */
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -206,7 +215,7 @@ static unsigned int SerializeHuffman2DImage(const ImageInfo *image_info,
       strip_size=byte_count[i];
     *length+=byte_count[i];
   }
-  buffer=MagickAllocateMemory(unsigned char *,strip_size);
+  buffer=MagickAllocateResourceLimitedMemory(unsigned char *,strip_size);
   if (buffer == (unsigned char *) NULL)
     {
       TIFFClose(tiff);
@@ -214,10 +223,10 @@ static unsigned int SerializeHuffman2DImage(const ImageInfo *image_info,
       ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
         (char *) NULL)
     }
-  *pixels=MagickAllocateMemory(unsigned char *,*length);
+  *pixels=MagickAllocateResourceLimitedMemory(unsigned char *,*length);
   if (*pixels == (unsigned char *) NULL)
     {
-      MagickFreeMemory(buffer);
+      MagickFreeResourceLimitedMemory(unsigned char *,buffer);
       TIFFClose(tiff);
       (void) LiberateTemporaryFile(filename);
       ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
@@ -234,7 +243,7 @@ static unsigned int SerializeHuffman2DImage(const ImageInfo *image_info,
     for (j=0; j < count; j++)
       *p++=buffer[j];
   }
-  MagickFreeMemory(buffer);
+  MagickFreeResourceLimitedMemory(unsigned char *,buffer);
   TIFFClose(tiff);
   (void) LiberateTemporaryFile(filename);
   return(True);
@@ -268,7 +277,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   for (i=0; i < length; i++)
     Ascii85Encode(image,(unsigned long) pixels[i]);
   Ascii85Flush(image);
-  MagickFreeMemory(pixels);
+  MagickFreeResourceLimitedMemory(unsigned char *,pixels);
   return(True);
 }
 
@@ -425,7 +434,7 @@ ModuleExport void RegisterPS3Image(void)
 %
 %    o pixels: the serialized indexes.
 %
-%    o length: the length of the pixels mamory area.
+%    o length: the length of the pixels memory area.
 %
 */
 static unsigned int SerializePseudoClassImage(const ImageInfo *image_info,
@@ -450,8 +459,8 @@ static unsigned int SerializePseudoClassImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   status=True;
-  *length=image->columns*image->rows;
-  *pixels=MagickAllocateMemory(unsigned char *, *length);
+  *length=MagickArraySize(image->columns,image->rows);
+  *pixels=MagickAllocateResourceLimitedMemory(unsigned char *, *length);
   if (*pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
   q=(*pixels);
@@ -474,7 +483,7 @@ static unsigned int SerializePseudoClassImage(const ImageInfo *image_info,
         }
   }
   if (status == False)
-    MagickFreeMemory(*pixels);
+    MagickFreeResourceLimitedMemory(unsigned char *,*pixels);
   return(status);
 }
 
@@ -506,7 +515,7 @@ static unsigned int SerializePseudoClassImage(const ImageInfo *image_info,
 %
 %    o pixels: the serialized image channels.
 %
-%    o length: the length of the pixels mamory area.
+%    o length: the length of the pixels memory area.
 %
 */
 static unsigned int SerializeMultiChannelImage(const ImageInfo *image_info,
@@ -528,8 +537,8 @@ static unsigned int SerializeMultiChannelImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   status=True;
-  *length=(image->colorspace == CMYKColorspace ? 4 : 3)*image->columns*image->rows;
-  *pixels=MagickAllocateMemory(unsigned char *, *length);
+  *length=(size_t) (image->colorspace == CMYKColorspace ? 4U : 3U)*MagickArraySize(image->columns,image->rows);
+  *pixels=MagickAllocateResourceLimitedMemory(unsigned char *, *length);
   if (*pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
 
@@ -567,7 +576,7 @@ static unsigned int SerializeMultiChannelImage(const ImageInfo *image_info,
         }
   }
   if (status == False)
-    MagickFreeMemory(*pixels);
+    MagickFreeResourceLimitedMemory(unsigned char *,*pixels);
   return(status);
 }
 
@@ -602,7 +611,7 @@ static unsigned int SerializeMultiChannelImage(const ImageInfo *image_info,
 %
 %    o pixels: the serialized image channels.
 %
-%    o length: the length of the pixels mamory area.
+%    o length: the length of the pixels memory area.
 %
 */
 static unsigned int SerializeSingleChannelImage(const ImageInfo *image_info,
@@ -636,7 +645,7 @@ static unsigned int SerializeSingleChannelImage(const ImageInfo *image_info,
   /* Padded columns are padded to byte boundary */
   padded_columns=((image->columns+pack-1)/pack)*pack;
   *length=padded_columns*image->rows/pack;
-  *pixels=MagickAllocateMemory(unsigned char *, *length);
+  *pixels=MagickAllocateResourceLimitedMemory(unsigned char *, *length);
   if (*pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
       image);
@@ -684,7 +693,7 @@ static unsigned int SerializeSingleChannelImage(const ImageInfo *image_info,
         }
   }
   if (status == False)
-    MagickFreeMemory(*pixels);
+    MagickFreeResourceLimitedMemory(unsigned char *,*pixels);
   return(status);
 }
 /*
@@ -808,28 +817,28 @@ static MagickPassFail WritePS3MaskImage(const ImageInfo *image_info,Image *image
   {
     case NoCompression:
       FormatString(buffer,
-        "currentfile %lu %lu "PS3_NoCompression" ByteStreamDecodeFilter\n",
+        "currentfile %lu %lu " PS3_NoCompression " ByteStreamDecodeFilter\n",
         image->columns, image->rows);
       break;
     case FaxCompression:
     default:
       FormatString(buffer,
-        "currentfile %lu %lu "PS3_FaxCompression" ByteStreamDecodeFilter\n",
+        "currentfile %lu %lu " PS3_FaxCompression " ByteStreamDecodeFilter\n",
         image->columns, image->rows);
       break;
     case RLECompression:
       FormatString(buffer,
-        "currentfile %lu %lu "PS3_RLECompression" ByteStreamDecodeFilter\n",
+        "currentfile %lu %lu " PS3_RLECompression " ByteStreamDecodeFilter\n",
         image->columns, image->rows);
       break;
     case LZWCompression:
       FormatString(buffer,
-        "currentfile %lu %lu "PS3_LZWCompression" ByteStreamDecodeFilter\n",
+        "currentfile %lu %lu " PS3_LZWCompression " ByteStreamDecodeFilter\n",
         image->columns, image->rows);
       break;
     case ZipCompression:
       FormatString(buffer,
-        "currentfile %lu %lu "PS3_ZipCompression" ByteStreamDecodeFilter\n",
+        "currentfile %lu %lu " PS3_ZipCompression " ByteStreamDecodeFilter\n",
         image->columns, image->rows);
       break;
   }
@@ -860,7 +869,7 @@ static MagickPassFail WritePS3MaskImage(const ImageInfo *image_info,Image *image
         for (i=0; i < length; i++)
           Ascii85Encode(image, (unsigned long)pixels[i]);
         Ascii85Flush(image);
-        MagickFreeMemory(pixels);
+        MagickFreeResourceLimitedMemory(unsigned char *,pixels);
       }
       break;
     case FaxCompression:
@@ -884,7 +893,7 @@ static MagickPassFail WritePS3MaskImage(const ImageInfo *image_info,Image *image
           for (i=0; i < length; i++)
             Ascii85Encode(image,(unsigned long) pixels[i]);
           Ascii85Flush(image);
-          MagickFreeMemory(pixels);
+          MagickFreeResourceLimitedMemory(unsigned char *,pixels);
         }
       }
       break;
@@ -896,7 +905,7 @@ static MagickPassFail WritePS3MaskImage(const ImageInfo *image_info,Image *image
         status=PackbitsEncode2Image(image,length,pixels,
           Ascii85WriteByteHook,(void *)NULL);
         Ascii85Flush(image);
-        MagickFreeMemory(pixels);
+        MagickFreeResourceLimitedMemory(unsigned char *,pixels);
       }
       break;
     case LZWCompression:
@@ -906,7 +915,7 @@ static MagickPassFail WritePS3MaskImage(const ImageInfo *image_info,Image *image
         Ascii85Initialize(image);
         status=LZWEncode2Image(image,length,pixels,Ascii85WriteByteHook,(void*)NULL);
         Ascii85Flush(image);
-        MagickFreeMemory(pixels);
+        MagickFreeResourceLimitedMemory(unsigned char *,pixels);
       }
       break;
     case ZipCompression:
@@ -917,7 +926,7 @@ static MagickPassFail WritePS3MaskImage(const ImageInfo *image_info,Image *image
         status=ZLIBEncode2Image(image,length,image_info->quality,pixels,
           Ascii85WriteByteHook,(void*)NULL);
         Ascii85Flush(image);
-        MagickFreeMemory(pixels);
+        MagickFreeResourceLimitedMemory(unsigned char *,pixels);
       }
       break;
   }
@@ -958,12 +967,12 @@ static MagickPassFail WritePS3MaskImage(const ImageInfo *image_info,Image *image
 %
 %  The format of the WritePS3Image method is:
 %
-%      unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
+%      MagickPassFail WritePS3Image(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method WritePS3Image return True if the image is printed.
-%      False is returned if the image file cannot be opened for printing.
+%    o status: Method WritePS3Image return MagickPass if the image is printed.
+%      MagickFail is returned if the image file cannot be opened for printing.
 %
 %    o image_info: Specifies a pointer to a ImageInfo structure.
 %
@@ -972,220 +981,214 @@ static MagickPassFail WritePS3MaskImage(const ImageInfo *image_info,Image *image
 %
 %
 */
-static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
+static MagickPassFail WritePS3Image(const ImageInfo *image_info,Image *image)
 {
-  static const char
-    * const PostscriptProlog[]=
-    {
-      "/ByteStreamDecodeFilter",
-      "{",
-      "  /z exch def",
-      "  /r exch def",
-      "  /c exch def",
-      "  /ASCII85Decode filter",
-      "  z "PS3_FaxCompression" eq",
-      "  {",
-      "    <<",
-      "      /K "CCITTParam,
-      "      /Columns c",
-      "      /Rows r",
-      "    >>",
-      "    /CCITTFaxDecode filter",
-      "  } if",
-      "  z "PS3_RLECompression" eq {/RunLengthDecode filter} if",
-      "  z "PS3_LZWCompression" eq {/LZWDecode filter} if",
-      "  z "PS3_ZipCompression" eq {/FlateDecode filter} if",
-      "  z "PS3_JPEGCompression" eq {/DCTDecode filter} if",
-      "} bind def",
-      "",
-      "/DirectClassImageDict",
-      "{",
-      "  colorspace "PS3_RGBColorspace" eq",
-      "  {",
-      "    /DeviceRGB setcolorspace",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent 8",
-      "      /DataSource pixel_stream",
-      "      /MultipleDataSources false",
-      "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
-      "      /Decode [0 1 0 1 0 1]",
-      "    >>",
-      "  }",
-      "  {",
-      "    /DeviceCMYK setcolorspace",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent 8",
-      "      /DataSource pixel_stream",
-      "      /MultipleDataSources false",
-      "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
-      "      /Decode",
-      /*
-        JPEG coder is used for JPEG compression. It compensates
-        for inverted CMYK, so use an inverted decode matrix for
-        JPEG compressed CMYK images.
-      */
-      "        compression "PS3_JPEGCompression" eq",
-      "        {[1 0 1 0 1 0 1 0]}",
-      "        {[0 1 0 1 0 1 0 1]}",
-      "        ifelse",
-      "    >>",
-      "  }",
-      "  ifelse",
-      "} bind def",
-      "",
-      "/PseudoClassImageDict",
-      "{",
-      "  % COLORS IN PSEUDO CLASS IMAGE",
-      "  currentfile buffer readline pop",
-      "  token pop /colors exch def pop",
-      "  colors 0 eq",
-      "  {",
-      "    % DEPTH OF GRAYSCALE",
-      "    currentfile buffer readline pop",
-      "    token pop /bits exch def pop",
-      "    /DeviceGray setcolorspace",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent bits",
-      "      /Decode [0 1]",
-      "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
-      "      /DataSource pixel_stream",
-      "    >>",
-      "  }",
-      "  {",
-      "    % RGB COLORMAP",
-      "    /colormap colors 3 mul string def",
-      "    % INDEXES",
-      "    currentfile /ASCII85Decode filter colormap readstring pop pop",
-      "    [ /Indexed /DeviceRGB colors 1 sub colormap ] setcolorspace",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent 8",
-      "      /Decode [0 255]",
-      "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
-      "      /DataSource pixel_stream",
-      "    >>",
-      "  }",
-      "  ifelse",
-      "} bind def",
-      "",
-      "/NonMaskedImageDict",
-      "{",
-      "  class "PS3_PseudoClass" eq",
-      "  { PseudoClassImageDict }",
-      "  { DirectClassImageDict }",
-      "  ifelse",
-      "} bind def",
-      "",
-      "/MaskedImageDict",
-      "{",
-      "  <<",
-      "    /ImageType 3",
-      "    /InterleaveType 3",
-      "    /DataDict NonMaskedImageDict",
-      "    /MaskDict",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent 1",
-      "      /DataSource mask_stream",
-      "      /MultipleDataSources false",
-      "      /ImageMatrix [ columns 0 0 rows neg 0 rows]",
-      "      /Decode [ 0 1 ]",
-      "    >>",
-      "  >>",
-      "} bind def",
-      "",
-      /*
-        Default procedure for image clipping does nothing. Image
-        will provide overriding ClipPath procdure if relevant.
-      */
-      "/ClipImage",
-      "{} def",
-      "",
-      "/DisplayImage",
-      "{",
-      "  /buffer 512 string def",
-      "",
-      "  % TRANSLATION",
-      "  currentfile buffer readline pop",
-      "  token pop /x exch def",
-      "  token pop /y exch def pop",
-      "  x y translate",
-      "",
-      "  % IMAGE SIZE AND FONT SIZE",
-      "  currentfile buffer readline pop",
-      "  token pop /x exch def",
-      "  token pop /y exch def pop",
-      "  currentfile buffer readline pop",
-      "  token pop /pointsize exch def pop",
-      (const char *) NULL
-    },
+  static const char PostscriptProlog[]=
+    "/ByteStreamDecodeFilter\n"
+    "{\n"
+    "  /z exch def\n"
+    "  /r exch def\n"
+    "  /c exch def\n"
+    "  /ASCII85Decode filter\n"
+    "  z " PS3_FaxCompression " eq\n"
+    "  {\n"
+    "    <<\n"
+    "      /K " CCITTParam "\n"
+    "      /Columns c\n"
+    "      /Rows r\n"
+    "    >>\n"
+    "    /CCITTFaxDecode filter\n"
+    "  } if\n"
+    "  z " PS3_RLECompression " eq {/RunLengthDecode filter} if\n"
+    "  z " PS3_LZWCompression " eq {/LZWDecode filter} if\n"
+    "  z " PS3_ZipCompression " eq {/FlateDecode filter} if\n"
+    "  z " PS3_JPEGCompression " eq {/DCTDecode filter} if\n"
+    "} bind def\n"
+    "\n"
+    "/DirectClassImageDict\n"
+    "{\n"
+    "  colorspace " PS3_RGBColorspace " eq\n"
+    "  {\n"
+    "    /DeviceRGB setcolorspace\n"
+    "    <<\n"
+    "      /ImageType 1\n"
+    "      /Width columns\n"
+    "      /Height rows\n"
+    "      /BitsPerComponent 8\n"
+    "      /DataSource pixel_stream\n"
+    "      /MultipleDataSources false\n"
+    "      /ImageMatrix [columns 0 0 rows neg 0 rows]\n"
+    "      /Decode [0 1 0 1 0 1]\n"
+    "    >>\n"
+    "  }\n"
+    "  {\n"
+    "    /DeviceCMYK setcolorspace\n"
+    "    <<\n"
+    "      /ImageType 1\n"
+    "      /Width columns\n"
+    "      /Height rows\n"
+    "      /BitsPerComponent 8\n"
+    "      /DataSource pixel_stream\n"
+    "      /MultipleDataSources false\n"
+    "      /ImageMatrix [columns 0 0 rows neg 0 rows]\n"
+    "      /Decode\n"
     /*
-      This hole in the PS prolog is for labels.
+      JPEG coder is used for JPEG compression. It compensates
+      for inverted CMYK, so use an inverted decode matrix for
+      JPEG compressed CMYK images.
     */
-    *PostscriptEpilog[]=
-    {
-      "  x y scale",
-      "",
-      "  % CLIPPING PATH",
-      "  currentfile buffer readline pop",
-      "  token pop /clipped exch def pop",
-      "",
-      "  % EPS",
-      "  currentfile buffer readline pop",
-      "  token pop /sp exch def pop",
-      "",
-      "  % IMAGE PIXEL SIZE",
-      "  currentfile buffer readline pop",
-      "  token pop /columns exch def",
-      "  token pop /rows exch def pop",
-      "",
-      "  % COLORSPACE (RGB/CMYK)",
-      "  currentfile buffer readline pop",
-      "  token pop /colorspace exch def pop",
-      "",
-      "  % TRANSPARENCY",
-      "  currentfile buffer readline pop",
-      "  token pop /alpha exch def pop",
-      "",
-      "  % STENCIL MASK?",
-      "  currentfile buffer readline pop",
-      "  token pop /stencil exch def pop",
-      "",
-      "  % IMAGE CLASS (DIRECT/PSEUDO)",
-      "  currentfile buffer readline pop",
-      "  token pop /class exch def pop",
-      "",
-      "  % COMPRESSION",
-      "  currentfile buffer readline pop",
-      "  token pop /compression exch def pop",
-      "",
-      "  % CLIP AND RENDER",
-      "  /pixel_stream currentfile columns rows compression",
-      "    ByteStreamDecodeFilter def",
-      "  clipped {ClipImage} if",
-      "  alpha stencil not and",
-      "  { MaskedImageDict mask_stream resetfile }",
-      "  { NonMaskedImageDict }",
-      "  ifelse",
-      "  stencil {0 setgray imagemask} {image} ifelse",
-      "  grestore",
-      "  sp {showpage} if",
-      "} bind def",
-      (const char *) NULL
-    };
+    "        compression " PS3_JPEGCompression " eq\n"
+    "        {[1 0 1 0 1 0 1 0]}\n"
+    "        {[0 1 0 1 0 1 0 1]}\n"
+    "        ifelse\n"
+    "    >>\n"
+    "  }\n"
+    "  ifelse\n"
+    "} bind def\n"
+    "\n"
+    "/PseudoClassImageDict\n"
+    "{\n"
+    "  % COLORS IN PSEUDO CLASS IMAGE\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /colors exch def pop\n"
+    "  colors 0 eq\n"
+    "  {\n"
+    "    % DEPTH OF GRAYSCALE\n"
+    "    currentfile buffer readline pop\n"
+    "    token pop /bits exch def pop\n"
+    "    /DeviceGray setcolorspace\n"
+    "    <<\n"
+    "      /ImageType 1\n"
+    "      /Width columns\n"
+    "      /Height rows\n"
+    "      /BitsPerComponent bits\n"
+    "      /Decode [0 1]\n"
+    "      /ImageMatrix [columns 0 0 rows neg 0 rows]\n"
+    "      /DataSource pixel_stream\n"
+    "    >>\n"
+    "  }\n"
+    "  {\n"
+    "    % RGB COLORMAP\n"
+    "    /colormap colors 3 mul string def\n"
+    "    % INDEXES\n"
+    "    currentfile /ASCII85Decode filter colormap readstring pop pop\n"
+    "    [ /Indexed /DeviceRGB colors 1 sub colormap ] setcolorspace\n"
+    "    <<\n"
+    "      /ImageType 1\n"
+    "      /Width columns\n"
+    "      /Height rows\n"
+    "      /BitsPerComponent 8\n"
+    "      /Decode [0 255]\n"
+    "      /ImageMatrix [columns 0 0 rows neg 0 rows]\n"
+    "      /DataSource pixel_stream\n"
+    "    >>\n"
+    "  }\n"
+    "  ifelse\n"
+    "} bind def\n"
+    "\n"
+    "/NonMaskedImageDict\n"
+    "{\n"
+    "  class " PS3_PseudoClass " eq\n"
+    "  { PseudoClassImageDict }\n"
+    "  { DirectClassImageDict }\n"
+    "  ifelse\n"
+    "} bind def\n"
+    "\n"
+    "/MaskedImageDict\n"
+    "{\n"
+    "  <<\n"
+    "    /ImageType 3\n"
+    "    /InterleaveType 3\n"
+    "    /DataDict NonMaskedImageDict\n"
+    "    /MaskDict\n"
+    "    <<\n"
+    "      /ImageType 1\n"
+    "      /Width columns\n"
+    "      /Height rows\n"
+    "      /BitsPerComponent 1\n"
+    "      /DataSource mask_stream\n"
+    "      /MultipleDataSources false\n"
+    "      /ImageMatrix [ columns 0 0 rows neg 0 rows]\n"
+    "      /Decode [ 0 1 ]\n"
+    "    >>\n"
+    "  >>\n"
+    "} bind def\n"
+    "\n"
+    /*
+      Default procedure for image clipping does nothing. Image
+      will provide overriding ClipPath procdure if relevant.
+    */
+    "/ClipImage\n"
+    "{} def\n"
+    "\n"
+    "/DisplayImage\n"
+    "{\n"
+    "  /buffer 512 string def\n"
+    "\n"
+    "  % TRANSLATION\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /x exch def\n"
+    "  token pop /y exch def pop\n"
+    "  x y translate\n"
+    "\n"
+    "  % IMAGE SIZE AND FONT SIZE\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /x exch def\n"
+    "  token pop /y exch def pop\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /pointsize exch def pop\n";
+
+  /*
+    This hole in the PS prolog is for labels.
+  */
+  static const char PostscriptEpilog[]=
+    "  x y scale\n"
+    "\n"
+    "  % CLIPPING PATH\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /clipped exch def pop\n"
+    "\n"
+    "  % EPS\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /sp exch def pop\n"
+    "\n"
+    "  % IMAGE PIXEL SIZE\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /columns exch def\n"
+    "  token pop /rows exch def pop\n"
+    "\n"
+    "  % COLORSPACE (RGB/CMYK)\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /colorspace exch def pop\n"
+    "\n"
+    "  % TRANSPARENCY\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /alpha exch def pop\n"
+    "\n"
+    "  % STENCIL MASK?\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /stencil exch def pop\n"
+    "\n"
+    "  % IMAGE CLASS (DIRECT/PSEUDO)\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /class exch def pop\n"
+    "\n"
+    "  % COMPRESSION\n"
+    "  currentfile buffer readline pop\n"
+    "  token pop /compression exch def pop\n"
+    "\n"
+    "  % CLIP AND RENDER\n"
+    "  /pixel_stream currentfile columns rows compression\n"
+    "    ByteStreamDecodeFilter def\n"
+    "  clipped {ClipImage} if\n"
+    "  alpha stencil not and\n"
+    "  { MaskedImageDict mask_stream resetfile }\n"
+    "  { NonMaskedImageDict }\n"
+    "  ifelse\n"
+    "  stencil {0 setgray imagemask} {image} ifelse\n"
+    "  grestore\n"
+    "  sp {showpage} if\n"
+    "} bind def\n";
 
   char
     buffer[MaxTextExtent],
@@ -1194,9 +1197,6 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     page_geometry[MaxTextExtent],
     **labels;
 
-  const char
-    * const *q;
-
   CompressionType
     compression;
 
@@ -1244,6 +1244,12 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     scene,
     text_size;
 
+  size_t
+    image_list_length;
+
+  Image
+    *clip_mask;
+
   /*
     Open output image file.
   */
@@ -1251,6 +1257,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  image_list_length=GetImageListLength(image);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
@@ -1280,6 +1287,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   }
   page=0;
   scene=0;
+  clip_mask = *ImageGetClipMask(image);
   do
   {
     page++;
@@ -1372,6 +1380,9 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     */
     if (page == 1)
       {
+#if defined(HAVE_CTIME_R)
+        char time_buf[26];
+#endif /* defined(HAVE_CTIME_R) */
         /* Postscript magic */
         if (LocaleCompare(image_info->magick,"PS3") == 0)
           (void) strlcpy(buffer,"%!PS-Adobe-3.0\n", sizeof(buffer));
@@ -1390,8 +1401,11 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
 
         /* File creation timestamp */
         timer=time((time_t *) NULL);
-        (void) localtime(&timer);
-        (void) strlcpy(date,ctime(&timer),MaxTextExtent);
+#if defined(HAVE_CTIME_R)
+        (void) strlcpy(date,ctime_r(&timer,time_buf),MaxTextExtent);
+#else
+        (void) strlcpy(date,ctime(&timer),MaxTextExtent); /* Thread-unsafe version */
+#endif /* defined(HAVE_CTIME_R) */
         date[strlen(date)-1]='\0';
         FormatString(buffer,"%%%%CreationDate: %.1024s\n",date);
         (void) WriteBlobString(image,buffer);
@@ -1448,18 +1462,14 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
               (void) strlcpy(buffer,"%%Pages: 1\n",sizeof(buffer));
             else
               FormatString(buffer,"%%%%Pages: %lu\n",(unsigned long)
-                GetImageListLength(image));
+                           image_list_length);
             (void) WriteBlobString(image,buffer);
           }
         (void) WriteBlobString(image,"%%EndComments\n");
 
         /* The static postscript procedures prolog. */
         (void)WriteBlobString(image,"%%BeginProlog\n");
-        for (q=PostscriptProlog; *q; q++)
-          {
-            (void) WriteBlobString(image,*q);
-            (void) WriteBlobByte(image,'\n');
-          }
+        (void) WriteBlob(image,sizeof(PostscriptProlog)-1,PostscriptProlog);
 
         /* One label line for each line in label string */
         attribute=GetImageAttribute(image,"label");
@@ -1481,11 +1491,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
           }
 
         /* The static postscript procedures epilog. */
-        for (q=PostscriptEpilog; *q; q++)
-          {
-            (void) WriteBlobString(image,*q);
-            (void) WriteBlobByte(image,'\n');
-          }
+        (void) WriteBlob(image,sizeof(PostscriptEpilog)-1,PostscriptEpilog);
         (void)WriteBlobString(image,"%%EndProlog\n");
       }
 
@@ -1526,13 +1532,13 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,"%%PageResources: font Helvetica\n");
 
     /* PS clipping path from Photoshop clipping path */
-    if ((image->clip_mask != (Image *) NULL) &&
-      (LocaleNCompare("8BIM:",image->clip_mask->magick_filename,5) == 0))
+    if ((clip_mask != (Image *) NULL) &&
+      (LocaleNCompare("8BIM:",clip_mask->magick_filename,5) == 0))
       {
         const ImageAttribute
           *attribute;
 
-        attribute=GetImageAttribute(image,image->clip_mask->magick_filename);
+        attribute=GetImageAttribute(image,clip_mask->magick_filename);
         if (attribute == (const ImageAttribute *) NULL)
           return(False);
         (void) WriteBlobString(image,attribute->value);
@@ -1598,13 +1604,13 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
       }
 
     /* Photoshop clipping path active? */
-    if ((image->clip_mask != (Image *) NULL) &&
-        (LocaleNCompare("8BIM:",image->clip_mask->magick_filename,5) == 0))
+    if ((clip_mask != (Image *) NULL) &&
+        (LocaleNCompare("8BIM:",clip_mask->magick_filename,5) == 0))
       (void) WriteBlobString(image,"true\n");
     else
       (void) WriteBlobString(image,"false\n");
 
-    /* Compression seems to take precedence over anyting */
+    /* Compression seems to take precedence over anything */
     if (compression == FaxCompression)
       (void) SetImageType(image, BilevelType);
 
@@ -1727,7 +1733,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
                 break;
             }
             Ascii85Flush(image);
-            MagickFreeMemory(pixels);
+            MagickFreeResourceLimitedMemory(unsigned char *,pixels);
           }
     }
     else
@@ -1796,7 +1802,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
                   break;
               }
               Ascii85Flush(image);
-              MagickFreeMemory(pixels);
+              MagickFreeResourceLimitedMemory(unsigned char *,pixels);
             }
         }
       else
@@ -1872,7 +1878,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
               break;
           }
           Ascii85Flush(image);
-          MagickFreeMemory(pixels);
+          MagickFreeResourceLimitedMemory(unsigned char *,pixels);
         }
 
     if (!status)
@@ -1905,7 +1911,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitorFormatted(scene,GetImageListLength(image),
+    status=MagickMonitorFormatted(scene,image_list_length,
                                   &image->exception,SaveImagesText,
                                   image->filename);
     if (status == False)
@@ -1927,8 +1933,8 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
-  CloseBlob(image);
-  return(True);
+  status &= CloseBlob(image);
+  return(status);
 }
 
 #if defined(HasZLIB)
@@ -1975,10 +1981,11 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
 %
 */
 
+static voidpf ZLIBAllocFunc(voidpf opaque, uInt items, uInt size) MAGICK_FUNC_MALLOC;
 static voidpf ZLIBAllocFunc(voidpf opaque, uInt items, uInt size)
 {
   ARG_NOT_USED(opaque);
-  return MagickMallocCleared((size_t) items*size);
+  return MagickMallocCleared(MagickArraySize(items,size));
 }
 static void ZLIBFreeFunc(voidpf opaque, voidpf address)
 {
@@ -2008,7 +2015,7 @@ static unsigned int ZLIBEncode2Image(Image *image,const size_t length,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   compressed_packets=(unsigned long) (1.001*length+12);
-  compressed_pixels=MagickAllocateMemory(unsigned char *,compressed_packets);
+  compressed_pixels=MagickAllocateResourceLimitedMemory(unsigned char *,compressed_packets);
   if (compressed_pixels == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
       (char *) NULL);
@@ -2030,11 +2037,14 @@ static unsigned int ZLIBEncode2Image(Image *image,const size_t length,
       compressed_packets=stream.total_out;
     }
   if (status)
-    ThrowBinaryException(CoderError,UnableToZipCompressImage,(char *) NULL)
+    {
+      (void) deflateEnd(&stream);
+      ThrowBinaryException(CoderError,UnableToZipCompressImage,(char *) NULL);
+    }
   else
     for (i=0; i < (long) compressed_packets; i++)
       (void) (*write_byte)(image,(magick_uint8_t)compressed_pixels[i],info);
-  MagickFreeMemory(compressed_pixels);
+  MagickFreeResourceLimitedMemory(unsigned char *,compressed_pixels);
   return(!status);
 }
 #else
