@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2022 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -44,6 +44,7 @@
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -499,7 +500,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   image->colormap[i].blue=ScaleCharToQuantum(sun_colormap[i]);
                 break;
               } while (1);
-            MagickFreeResourceLimitedMemory(sun_colormap);
+            MagickFreeResourceLimitedMemory(unsigned char *,sun_colormap);
             if (MagickFail == status)
               ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
             if (logging)
@@ -525,7 +526,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (ReadBlob(image,sun_info.maplength,(char *) sun_colormap) !=
                 sun_info.maplength)
               status = MagickFail;
-            MagickFreeResourceLimitedMemory(sun_colormap);
+            MagickFreeResourceLimitedMemory(unsigned char *,sun_colormap);
             if (MagickFail == status)
               ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
             if (logging)
@@ -623,7 +624,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if ((count=ReadBlob(image,sun_info.length,(char *) sun_data))
           != sun_info.length)
         {
-          MagickFreeResourceLimitedMemory(sun_data);
+          MagickFreeResourceLimitedMemory(unsigned char *,sun_data);
           ThrowReaderException(CorruptImageError,UnableToReadImageData,image);
         }
       sun_pixels=sun_data;
@@ -636,15 +637,15 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                                          bytes_per_image+image->rows);
           if (sun_pixels == (unsigned char *) NULL)
             {
-              MagickFreeResourceLimitedMemory(sun_data);
+              MagickFreeResourceLimitedMemory(unsigned char *,sun_data);
               ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
                                    image);
             }
           status &= DecodeImage(sun_data,sun_data_length,sun_pixels,bytes_per_image);
-          MagickFreeResourceLimitedMemory(sun_data);
+          MagickFreeResourceLimitedMemory(unsigned char *,sun_data);
           if (status != MagickPass)
             {
-              MagickFreeResourceLimitedMemory(sun_pixels);
+              MagickFreeResourceLimitedMemory(unsigned char *,sun_pixels);
               ThrowReaderException(CorruptImageError,UnableToRunlengthDecodeImage,image);
             }
         }
@@ -782,7 +783,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     break;
             }
         }
-      MagickFreeResourceLimitedMemory(sun_pixels);
+      MagickFreeResourceLimitedMemory(unsigned char *,sun_pixels);
       if (EOFBlob(image))
         {
           ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,
@@ -855,7 +856,7 @@ ModuleExport void RegisterSUNImage(void)
   entry->magick=(MagickHandler) IsSUN;
   entry->description="SUN Rasterfile (1 bit)";
   entry->module="SUN";
-  entry->stealth=MagickTrue;
+  entry->stealth=MagickTrue; /* Don't list in '-list format' output */
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("IM8");
@@ -864,7 +865,7 @@ ModuleExport void RegisterSUNImage(void)
   entry->magick=(MagickHandler) IsSUN;
   entry->description="SUN Rasterfile (8 bit)";
   entry->module="SUN";
-  entry->stealth=MagickTrue;
+  entry->stealth=MagickTrue; /* Don't list in '-list format' output */
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("IM24");
@@ -873,7 +874,7 @@ ModuleExport void RegisterSUNImage(void)
   entry->magick=(MagickHandler) IsSUN;
   entry->description="SUN Rasterfile (24 bit)";
   entry->module="SUN";
-  entry->stealth=MagickTrue;
+  entry->stealth=MagickTrue; /* Don't list in '-list format' output */
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("RAS");
@@ -1229,7 +1230,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
                     break;
             }
         }
-      MagickFreeResourceLimitedMemory(pixels);
+      MagickFreeResourceLimitedMemory(unsigned char *,pixels);
       if (image->next == (Image *) NULL)
         break;
       image=SyncNextImageInList(image);
