@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2018 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -52,6 +52,7 @@
 #include "magick/render.h"
 #include "magick/texture.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 typedef enum
 {
@@ -92,7 +93,7 @@ static void readln(Image *image, int *pch)
   else
     ch=' ';
 
-  while (ch != 10 && ch != 13 && ch != EOF)
+  while (ch != 10 /* LF */ && ch != 13 /* CR */ && ch != EOF)
     {
       ch = ReadBlobByte(image);
     }
@@ -131,6 +132,7 @@ static long ReadInt(Image *image, int *pch)
       ch = ReadBlobByte(image);
       if (ch == EOF)
         return (n);
+      ch &= 0xff;
       digits++;
     }
 
@@ -200,7 +202,7 @@ static TXT_TYPE IsTXT(const unsigned char *magick,const size_t length)
       buffer[MaxTextExtent];
 
     (void) memset((void *)buffer,0,MaxTextExtent);
-    (void) memcpy((void *)buffer,(const void *)magick,Min(MaxTextExtent,length));
+    (void) memcpy((void *)buffer,(const void *)magick,Min(MaxTextExtent-1,length));
 
     if (!strncmp(buffer,"# ImageMagick pixel enumeration:",32))
       return IMAGEMAGICK_TXT;
@@ -328,7 +330,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
 
-  logging = IsEventLogging();
+  logging = IsEventLogged(CoderEvent);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
@@ -341,10 +343,10 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   if (txt_subformat != NO_TXT)
     {
-#define ThrowNOTXTReaderException(code_,reason_,image_) \
-      do {                                              \
-        MagickFreeMemory(BImgBuff);                     \
-        ThrowReaderException(code_,reason_,image_);     \
+#define ThrowNOTXTReaderException(code_,reason_,image_)                 \
+      do {                                                              \
+        MagickFreeResourceLimitedMemory(unsigned char *,BImgBuff);      \
+        ThrowReaderException(code_,reason_,image_);                     \
       } while (0);
 
       unsigned
@@ -630,7 +632,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   packet_size=40.0;  /* Max characters in a row */
 
                 double
-                  number_pixels=image->columns*image->rows;
+                  number_pixels=(double) image->columns*image->rows;
 
                 ratio = (((double) number_pixels*packet_size)/file_size);
 
@@ -649,10 +651,10 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /* Assure that all image pixels are initialized to black */
         SetImage(image,OpaqueOpacity);
 
-        BImgBuff = MagickAllocateArray(unsigned char *,
-                                       (size_t)(x+1),
-                                       ( ((image->matte) ? 4 : 3)
-                                         * NumOfPlanes/8));
+        BImgBuff = MagickAllocateResourceLimitedClearedArray(unsigned char *,
+                                                             ((size_t)x+1),
+                                                             ((size_t)((image->matte) ? 4 : 3)
+                                                              * NumOfPlanes/8));
         WImgBuff = (magick_uint16_t *)BImgBuff;
         DImgBuff = (magick_uint32_t *)BImgBuff;
         if (BImgBuff == NULL)
@@ -769,11 +771,11 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
                 if (image->matte)
                   (void)ImportImagePixelArea(image,RGBAQuantum,NumOfPlanes,
-                                             BImgBuff + 4*x_min*(NumOfPlanes/8),
+                                             BImgBuff + (size_t)4*x_min*(NumOfPlanes/8),
                                              &import_options,0);
                 else
                   (void)ImportImagePixelArea(image,RGBQuantum,NumOfPlanes,
-                                             BImgBuff + 3*x_min*(NumOfPlanes/8),
+                                             BImgBuff + (size_t)3*x_min*(NumOfPlanes/8),
                                              &import_options,0);
                 if (!SyncImagePixels(image))
                   break;
@@ -854,11 +856,11 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (image->matte)
                   (void)ImportImagePixelArea(image, RGBAQuantum, NumOfPlanes,
-                                             BImgBuff + 4*x_min*(NumOfPlanes/8),
+                                             BImgBuff + (size_t)4*x_min*(NumOfPlanes/8),
                                              &import_options, 0);
                 else
                   (void)ImportImagePixelArea(image, RGBQuantum, NumOfPlanes,
-                                             BImgBuff + 3*x_min*(NumOfPlanes/8),
+                                             BImgBuff + (size_t)3*x_min*(NumOfPlanes/8),
                                              &import_options, 0);
                 if (!SyncImagePixels(image))
                   {
@@ -870,7 +872,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
               }
           }
         /* Note that DImgBuff and WImgBuff point to BImgBuff */
-        MagickFreeMemory(BImgBuff);
+        MagickFreeResourceLimitedMemory(unsigned char *,BImgBuff);
       } while(!EOFBlob(image) && NextImagePos>0);
 
       goto FINISH_TXT;
@@ -953,7 +955,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
           *clone_info;
 
         clone_info=CloneImageInfo(image_info);
-        clone_info->blob=(void *) NULL;
+        clone_info->blob=(_BlobInfoPtr_) NULL;
         clone_info->length=0;
         (void) strlcpy(clone_info->filename,image_info->texture,MaxTextExtent);
         texture=ReadImage(clone_info,exception);
@@ -1034,6 +1036,8 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (p == (char *) NULL)
           break;
 
+        StopTimer(&image->timer);
+
         if (logging)
           (void)LogMagickEvent(CoderEvent,GetMagickModule(),
                                "page %ld scene %ld ",page_num, image->scene);
@@ -1310,6 +1314,6 @@ static unsigned int WriteTXTImage(const ImageInfo *image_info,Image *image)
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
       image=image->previous;
-  CloseBlob(image);
-  return (MagickTrue);
+  status &= CloseBlob(image);
+  return (status);
 }
