@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2018 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2003 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -17,6 +17,7 @@
 #include "magick/studio.h"
 #include "magick/analyze.h"
 #include "magick/color.h"
+#include "magick/log.h"
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
 #include "magick/pixel_iterator.h"
@@ -71,6 +72,9 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   unsigned long
     row_count=0;
 
+  MagickBool
+    monitor_active;
+
   PixelPacket
     corners[3];
 
@@ -80,11 +84,15 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
 
+  monitor_active=MagickMonitorActive();
   bounds.width=0;
   bounds.height=0;
   bounds.x=(long) image->columns;
   bounds.y=(long) image->rows;
 
+  if ((image->columns == 0) || (image->rows == 0))
+    return(bounds);
+
   (void) AcquireOnePixelByReference(image,&corners[0],0,0,exception);
   (void) AcquireOnePixelByReference(image,&corners[1],(long) image->columns-1,0,exception);
   (void) AcquireOnePixelByReference(image,&corners[2],0,(long) image->rows-1,exception);
@@ -188,14 +196,24 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
               }
         }
 
+      if (monitor_active)
+        {
+          unsigned long
+            thread_row_count;
+
 #if defined(HAVE_OPENMP)
 #  pragma omp atomic
 #endif
-      row_count++;
-      if (QuantumTick(row_count,image->rows))
-        if (!MagickMonitorFormatted(row_count,image->rows,exception,
-                                    GetImageBoundingBoxText,image->filename))
-          thread_status=MagickFail;
+          row_count++;
+#if defined(HAVE_OPENMP)
+#  pragma omp flush (row_count)
+#endif
+          thread_row_count=row_count;
+          if (QuantumTick(thread_row_count,image->rows))
+            if (!MagickMonitorFormatted(thread_row_count,image->rows,exception,
+                                        GetImageBoundingBoxText,image->filename))
+              thread_status=MagickFail;
+        }
 
 #if defined(HAVE_OPENMP)
 #  pragma omp critical (GM_GetImageBoundingBox)
@@ -219,11 +237,10 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
 #endif
         }
     }
-  if ((bounds.width != 0) || (bounds.height != 0))
-    {
-      bounds.width-=(bounds.x-1);
-      bounds.height-=(bounds.y-1);
-    }
+  if (bounds.width != 0)
+    bounds.width-=(bounds.x-1);
+  if (bounds.height != 0)
+    bounds.height-=(bounds.y-1);
   if (bounds.x < 0)
     bounds.x=0;
   if (bounds.y < 0)
@@ -239,6 +256,10 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
       bounds.x=0;
       bounds.y=0;
     }
+  if (IsEventLogged(TransformEvent))
+    (void) LogMagickEvent(TransformEvent,GetMagickModule(),
+                          "Bounding Box: %lux%lu%+ld%+ld",
+                          bounds.width, bounds.height, bounds.x, bounds.y);
 
   return(bounds);
 }
@@ -276,28 +297,28 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
 %
 %
 */
+#if MaxMap == MaxRGB
 static inline unsigned char MinimumDepthForValue(const Quantum quantum)
 {
   register unsigned int
     depth,
     scale;
 
-  for (depth=1 ; depth < MaxRGB; depth++)
+  for (depth=1 ; depth <= (unsigned int) QuantumDepth; depth++)
     {
       scale=MaxRGB / (MaxRGB >> (QuantumDepth-depth));
       if (quantum == scale*(quantum/scale))
         break;
     }
 
-  return depth;
+  return (unsigned char) depth;
 }
-#if MaxMap == MaxRGB
 static magick_uint8_t* AllocateDepthMap(void)
 {
   magick_uint8_t
     *map;
 
-  map = MagickAllocateArray(unsigned char *, MaxMap+1, sizeof(magick_uint8_t));
+  map = MagickAllocateResourceLimitedArray(unsigned char *, MaxMap+1, sizeof(magick_uint8_t));
   if (map != (unsigned char *) NULL)
     {
       unsigned int
@@ -449,7 +470,7 @@ MagickExport unsigned long GetImageDepth(const Image *image,
                                   image->rows,image,exception);
     }
 
-  MagickFreeMemory(map);
+  MagickFreeResourceLimitedMemory(magick_uint8_t *,map);
 
   return depth;
 }
