@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2017 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2003 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -22,6 +22,7 @@
 #include "magick/color.h"
 #include "magick/color_lookup.h"
 #include "magick/constitute.h"
+#include "magick/describe.h"
 #include "magick/enum_strings.h"
 #include "magick/magick.h"
 #include "magick/pixel_cache.h"
@@ -159,6 +160,8 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
                            image->error.normalized_mean_error,
                            image->error.normalized_maximum_error);
           }
+      if(image->matte==MagickTrue)
+          (void) fputs("+opacity ",file);
       (void) fprintf(file,"%u-bit ",image->depth);
       if (GetBlobSize(image) != 0)
         {
@@ -174,7 +177,8 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
         least six times the timer's resolution (typically 0.01 on
         Unix).
       */
-      if (!(image->ping) && (elapsed_time >= GetTimerResolution()*6))
+      if (!(image->ping) && (elapsed_time > 0.001) &&
+          (elapsed_time >= GetTimerResolution()*6))
         {
           pixels_per_second=(magick_int64_t) ((double) rows*columns/
                                               elapsed_time);
@@ -434,10 +438,14 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
       char
         tuple[MaxTextExtent];
 
+      MagickBool
+        found_transparency;
+
       register const PixelPacket
         *p;
 
       p=(PixelPacket *) NULL;
+      found_transparency = MagickFalse;
       for (y=0; y < image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -446,13 +454,16 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
           for (x=0; x < image->columns; x++)
             {
               if (p->opacity == TransparentOpacity)
-                break;
+                {
+                  found_transparency=MagickTrue;
+                  break;
+                }
               p++;
             }
           if (x < image->columns)
             break;
         }
-      if ((x < image->columns) || (y < image->rows))
+      if (found_transparency)
         {
           GetColorTuple(p,image->depth,image->matte,False,tuple);
           (void) fprintf(file,"  Opacity: %.1024s\t",tuple);
@@ -724,8 +735,8 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
             }
           i++;
           (void) fprintf(file,"    %.1024s:\n",tag);
-          length=profile[i++] << 8;
-          length|=profile[i++];
+          length=(size_t) profile[i++] << 8;
+          length|=(size_t) profile[i++];
           length=Min(length,profile_length-i);
           text=MagickAllocateMemory(char *,length+1);
           if (text != (char *) NULL)
