@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003 - 2023 GraphicsMagick Group
+  Copyright (C) 2003-2023 GraphicsMagick Group
   Copyright (C) 2002 ImageMagick Studio
   Copyright 1991-1999 E. I. du Pont de Nemours and Company
 
@@ -942,7 +942,7 @@ typedef struct _ImageInfo
   MagickBool
     dither,                  /* If true, dither image while writing */
     monochrome,              /* If true, use monochrome format */
-    progress;                /* If true, show progress indication */
+    progress;                /* If true, show progress indication (for X11 commands) */
 
   ColorspaceType
     colorspace;              /* Colorspace representations of image pixels */
