@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2017 GraphicsMagick Group
+% Copyright (C) 2003 - 2022 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -81,130 +81,7 @@ typedef enum
 static const char
   *BuiltInPath="[Built In]";
 
-static const struct
-{
-  char
-    *magick,
-    *name;
-}
-ModuleAliases[] =
-{
-#define MODULEALIAS(magick,name) {magick,name}
-  MODULEALIAS("3FR","DCRAW"),
-  MODULEALIAS("8BIM","META"),
-  MODULEALIAS("8BIMTEXT","META"),
-  MODULEALIAS("8BIMWTEXT","META"),
-  MODULEALIAS("APP1","META"),
-  MODULEALIAS("APP1JPEG","META"),
-  MODULEALIAS("ARW","DCRAW"),
-  MODULEALIAS("B","GRAY"),
-  MODULEALIAS("BIE","JBIG"),
-  MODULEALIAS("BIGTIFF","TIFF"),
-  MODULEALIAS("BMP2","BMP"),
-  MODULEALIAS("BMP3","BMP"),
-  MODULEALIAS("C","GRAY"),
-  MODULEALIAS("CAL","CALS"),
-  MODULEALIAS("CIN","CINEON"),
-  MODULEALIAS("CMYKA","CMYK"),
-  MODULEALIAS("CR2","DCRAW"),
-  MODULEALIAS("CRW","DCRAW"),
-  MODULEALIAS("CUR","ICON"),
-  MODULEALIAS("DCR","DCRAW"),
-  MODULEALIAS("DCX","PCX"),
-  MODULEALIAS("DNG","DCRAW"),
-  MODULEALIAS("EPDF","PDF"),
-  MODULEALIAS("EPI","PS"),
-  MODULEALIAS("EPS","PS"),
-  MODULEALIAS("EPS2","PS2"),
-  MODULEALIAS("EPS3","PS3"),
-  MODULEALIAS("EPSF","PS"),
-  MODULEALIAS("EPSI","PS"),
-  MODULEALIAS("EPT2","EPT"),
-  MODULEALIAS("EPT3","EPT"),
-  MODULEALIAS("ERF","DCRAW"),
-  MODULEALIAS("EXIF","META"),
-  MODULEALIAS("FILE","URL"),
-  MODULEALIAS("FRACTAL","PLASMA"),
-  MODULEALIAS("FTP","URL"),
-  MODULEALIAS("G","GRAY"),
-  MODULEALIAS("G3","FAX"),
-  MODULEALIAS("GIF87","GIF"),
-  MODULEALIAS("GRANITE","LOGO"),
-  MODULEALIAS("GRAYA","GRAY"),
-  MODULEALIAS("GROUP4RAW","TIFF"),
-  MODULEALIAS("H","LOGO"),
-  MODULEALIAS("HTM","HTML"),
-  MODULEALIAS("HTTP","URL"),
-  MODULEALIAS("ICB","TGA"),
-  MODULEALIAS("ICC","META"),
-  MODULEALIAS("ICM","META"),
-  MODULEALIAS("ICO","ICON"),
-  MODULEALIAS("IMAGE","LOGO"),
-  MODULEALIAS("IPTC","META"),
-  MODULEALIAS("IPTCTEXT","META"),
-  MODULEALIAS("IPTCWTEXT","META"),
-  MODULEALIAS("J2C","JP2"),
-  MODULEALIAS("JBG","JBIG"),
-  MODULEALIAS("JNG","PNG"),
-  MODULEALIAS("JPC","JP2"),
-  MODULEALIAS("JPG","JPEG"),
-  MODULEALIAS("K","GRAY"),
-  MODULEALIAS("K25","DCRAW"),
-  MODULEALIAS("KDC","DCRAW"),
-  MODULEALIAS("LOCALEC","LOCALE"),
-  MODULEALIAS("LOCALEH","LOCALE"),
-  MODULEALIAS("LOCALEMC","LOCALE"),
-  MODULEALIAS("M","GRAY"),
-  MODULEALIAS("M2V","MPEG"),
-  MODULEALIAS("MEF","DCRAW"),
-  MODULEALIAS("MNG","PNG"),
-  MODULEALIAS("MPG","MPEG"),
-  MODULEALIAS("MRW","DCRAW"),
-  MODULEALIAS("NEF","DCRAW"),
-  MODULEALIAS("NETSCAPE","LOGO"),
-  MODULEALIAS("O","GRAY"),
-  MODULEALIAS("ORF","DCRAW"),
-  MODULEALIAS("P7","PNM"),
-  MODULEALIAS("PAL","UYVY"),
-  MODULEALIAS("PAM","PNM"),
-  MODULEALIAS("PATTERN","LOGO"),
-  MODULEALIAS("PBM","PNM"),
-  MODULEALIAS("PCDS","PCD"),
-  MODULEALIAS("PCT","PICT"),
-  MODULEALIAS("PEF","DCRAW"),
-  MODULEALIAS("PFA","TTF"),
-  MODULEALIAS("PFB","TTF"),
-  MODULEALIAS("PGM","PNM"),
-  MODULEALIAS("PGX","JP2"),
-  MODULEALIAS("PICON","XPM"),
-  MODULEALIAS("PM","XPM"),
-  MODULEALIAS("PNG24","PNG"),
-  MODULEALIAS("PNG32","PNG"),
-  MODULEALIAS("PNG8","PNG"),
-  MODULEALIAS("PPM","PNM"),
-  MODULEALIAS("PTIF","TIFF"),
-  MODULEALIAS("R","GRAY"),
-  MODULEALIAS("RAF","DCRAW"),
-  MODULEALIAS("RAS","SUN"),
-  MODULEALIAS("RGBA","RGB"),
-  MODULEALIAS("ROSE","LOGO"),
-  MODULEALIAS("SHTML","HTML"),
-  MODULEALIAS("SR2","DCRAW"),
-  MODULEALIAS("SRF","DCRAW"),
-  MODULEALIAS("SVGZ","SVG"),
-  MODULEALIAS("TEXT","TXT"),
-  MODULEALIAS("TIF","TIFF"),
-  MODULEALIAS("VDA","TGA"),
-  MODULEALIAS("VST","TGA"),
-  MODULEALIAS("X3F","DCRAW"),
-  MODULEALIAS("XMP","META"),
-  MODULEALIAS("XTRNARRAY","XTRN"),
-  MODULEALIAS("XTRNBLOB","XTRN"),
-  MODULEALIAS("XTRNFILE","XTRN"),
-  MODULEALIAS("XTRNIMAGE","XTRN"),
-  MODULEALIAS("XV","VIFF"),
-  MODULEALIAS("Y","GRAY")
-};
+#include "magick/module_aliases.h"
 
 /*
   Coder module list
@@ -308,15 +185,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DestroyMagickResources() destroys the resource environment.
+%  DestroyMagickModules() destroys the modules environment.
 %
-%  The format of the DestroyMagickResources() method is:
+%  The format of the DestroyMagickModules() method is:
 %
-%      DestroyMagickResources(void)
+%      DestroyMagickModules(void)
 %
 %
 */
-MagickExport void
+void
 DestroyMagickModules(void)
 {
   DestroyModuleInfo();
@@ -371,7 +248,7 @@ DestroyModuleInfoEntry(ModuleInfo *entry)
   MagickFreeMemory(entry);
 }
 
-MagickExport void
+void
 DestroyModuleInfo(void)
 {
   CoderInfo
@@ -463,7 +340,7 @@ DestroyModuleInfo(void)
 %      arguments.
 %
 */
-MagickExport MagickPassFail
+MagickPassFail
 ExecuteModuleProcess(const char *tag,Image **image,
                      const int argc,char **argv)
 {
@@ -526,12 +403,26 @@ ExecuteModuleProcess(const char *tag,Image **image,
     method=(unsigned int (*)(Image **,const int,char **))
       lt_dlsym(handle,method_name);
 
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "Invoking \"%.1024s\" filter module",tag);
 
     /* Execute module method */
     if (method != (unsigned int (*)(Image **,const int,char **)) NULL)
-      status=(*method)(image,argc,argv);
+      {
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Invoking \"%.1024s\" filter module",tag);
+        status=(*method)(image,argc,argv);
+      }
+    else
+      {
+        char
+          message[MaxTextExtent];
+
+        FormatString(message,
+                     "Method name \"%.1024s\" was not found in module \"%.1024s\"!",
+                     method_name, tag);
+        ThrowException(&(*image)->exception,ModuleError,UnableToLoadModule,
+                       message);
+        status=MagickFail;
+      }
 
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       "Returned from \"%.1024s\" filter module",tag);
@@ -698,7 +589,7 @@ FindMagickModule(const char *filename,MagickModuleType module_type,
       return MagickFail;
     }
 
-  if (IsEventLogging())
+  if (IsEventLogged(ConfigureEvent))
     {
       char
         list_seperator[2],
@@ -710,7 +601,7 @@ FindMagickModule(const char *filename,MagickModuleType module_type,
         {
           if (search_path)
             (void) ConcatenateString(&search_path,list_seperator);
-          (void) ConcatenateString(&search_path,
+          (void) ConcatenateString(&search_path,(const char* )
                                    MagickMapDereferenceIterator(path_map_iterator,0));
         }
 
@@ -874,11 +765,8 @@ GetModuleList(ExceptionInfo *exception)
   unsigned long
     max_entries;
 
-  if (InitializeModuleSearchPath(MagickCoderModule,exception) == MagickFail)
-    return ((char **) NULL);
-
   max_entries=MAX_MODULES;
-  modules=MagickAllocateMemory(char **,(max_entries+1)*sizeof(char *));
+  modules=MagickAllocateArray(char **,((size_t) max_entries+1),sizeof(char *));
   if (modules == (char **) NULL)
     return((char **) NULL);
   modules[0]=(char *) NULL;
@@ -919,7 +807,7 @@ GetModuleList(ExceptionInfo *exception)
 %
 %
 */
-MagickExport void
+void
 InitializeMagickModules(void)
 {
   ExceptionInfo
@@ -1081,17 +969,17 @@ InitializeModuleSearchPath(MagickModuleType module_type,
             buffer[MaxTextExtent];
 
           const char
-            *seperator;
+            *separator;
 
-          int
+          size_t
             length;
 
           MagickBool
             skip=MagickFalse;
 
-          seperator = strchr(start,DirectoryListSeparator);
-          if (seperator)
-            length=seperator-start;
+          separator = strchr(start,DirectoryListSeparator);
+          if (separator)
+            length=separator-start;
           else
             length=end-start;
           if (length > MaxTextExtent-1)
@@ -1141,7 +1029,7 @@ InitializeModuleSearchPath(MagickModuleType module_type,
           if (skip == MagickFalse)
             {
               if (buffer[length-1] != DirectorySeparator[0])
-                (void) strcat(buffer,DirectorySeparator);
+                (void) strlcat(buffer,DirectorySeparator,sizeof(buffer));
               AddModulePath(path_map,&path_index,buffer,exception);
             }
           start += length+1;
@@ -1154,7 +1042,7 @@ InitializeModuleSearchPath(MagickModuleType module_type,
     /*
       Search hard coded paths.
     */
-    char
+    const char
       *module_directory=NULL;
 
     switch (module_type)
@@ -1279,7 +1167,7 @@ InitializeModuleSearchPath(MagickModuleType module_type,
   /*
     Search current directory.
   */
-  strcpy(path,"");
+  strlcpy(path,"",sizeof(path));
   AddModulePath(path_map,&path_index,path,exception);
 
   return (status);
@@ -1322,7 +1210,7 @@ InitializeModuleSearchPath(MagickModuleType module_type,
 %
 %
 */
-MagickExport const ModuleInfo *
+const ModuleInfo *
 GetModuleInfo(const char *name,ExceptionInfo *exception)
 {
   const ModuleInfo
@@ -1390,7 +1278,7 @@ GetModuleInfo(const char *name,ExceptionInfo *exception)
 %
 %
 */
-MagickExport unsigned int
+unsigned int
 ListModuleInfo(FILE *file,ExceptionInfo *exception)
 {
   register const ModuleInfo
@@ -1457,7 +1345,7 @@ ListModuleInfo(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-MagickExport MagickPassFail
+MagickPassFail
 OpenModule(const char *module,ExceptionInfo *exception)
 {
   {
@@ -1618,7 +1506,7 @@ OpenModule(const char *module,ExceptionInfo *exception)
 %    o exception: Return any errors or warnings in this structure.
 %
 */
-MagickExport MagickPassFail
+MagickPassFail
 OpenModules(ExceptionInfo *exception)
 {
   (void) GetMagickInfo((char *) NULL,exception);
@@ -1833,11 +1721,10 @@ ReadModuleConfigureFile(const char *basename,
                 Allocate memory for the module list.
               */
               in_entry=MagickTrue;
-              module_info=MagickAllocateMemory(ModuleInfo *,sizeof(ModuleInfo));
+              module_info=MagickAllocateClearedMemory(ModuleInfo *,sizeof(ModuleInfo));
               if (module_info == (ModuleInfo *) NULL)
                 MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
                                   UnableToAllocateModuleInfo);
-              (void) memset(module_info,0,sizeof(ModuleInfo));
               module_info->path=AcquireString(path);
               module_info->signature=MagickSignature;
               if (module_list == (ModuleInfo *) NULL)
@@ -2055,11 +1942,10 @@ SetCoderInfo(const char *tag)
     *entry;
 
   assert(tag != (const char *) NULL);
-  entry=MagickAllocateMemory(CoderInfo *,sizeof(CoderInfo));
+  entry=MagickAllocateClearedMemory(CoderInfo *,sizeof(CoderInfo));
   if (entry == (CoderInfo *) NULL)
     MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
       UnableToAllocateModuleInfo);
-  (void) memset(entry,0,sizeof(CoderInfo));
   entry->tag=AcquireString(tag);
   entry->signature=MagickSignature;
   return(entry);
